// Analyzer.cpp: implementation of the Analyzer class.
//
//////////////////////////////////////////////////////////////////////

#include "Analyzer.h"
#include "MinimaxTree.h"
#include <ctime>
#ifdef DEBUG2
#include <fstream>
#include <string>
#endif
using namespace std;

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

Analyzer::Analyzer(int plyLimit, int nodeLimit)
{
	this->plyLimit=plyLimit;
	this->nodeLimit=nodeLimit;

	timeMode=TotalTime;
	totalTime=300;
	increTime=0;
	moveNum=40;
	timer.Reset();
	timer.CountDown(totalTime);

	moveCounter=0;

	lastTwoConfigs[0]=NULL;
	lastTwoConfigs[1]=NULL;
}

Analyzer::~Analyzer()
{
	if(lastTwoConfigs[0]!=NULL)
		delete lastTwoConfigs[0];
	if(lastTwoConfigs[1]!=NULL)
		delete lastTwoConfigs[1];
}


void Analyzer::setTimeMode(TimeMode timeMode,int totalTime,int increTime,int moveNum)
{
	this->timeMode=timeMode;
	this->totalTime=totalTime;
	this->increTime=increTime;
	this->moveNum=moveNum;
	timer.Reset();
	if(timeMode==TotalTime||timeMode==IncreTime)
	{
		timer.CountDown(totalTime);
	}
}

//allocate time for each move
int Analyzer::timeAlloc()
{
	int timeAllocated;
	switch(timeMode)
	{
		case TotalTime:
		{
			if(moveCounter>=moveNum)
			{
				moveCounter=0;
			}
			timeAllocated=timer.Remains()/(moveNum-moveCounter);
			break;
		}
		case IncreTime:
		{
			if(moveCounter>=70)
			{
				moveCounter=0;
			}
			timeAllocated=increTime+timer.Remains()/(70-moveCounter); //70 is the expected move number of whole
			break;
		}
		case PlyTime:
		{
			timeAllocated=increTime;
			break;
		}
		default:
		{
			timeAllocated=12; //should never be reached
		}
	}

	return (int)(timeAllocated*2);
}

		

MOVE Analyzer::plyGen(Configuration* curConfig)
{
//	clock_t tStart;
//	clock_t tEnd;
//	tStart=clock();
#ifdef DEBUG1
	ConfigList* N_Children=curConfig->successors();

	ofstream debug2("test1.txt");
	ConfigList::iterator d_it=N_Children->begin();
	while(d_it != N_Children->end())
	{
		for(int i=0;i<8;i++)
		{
			for(int j=0;j<8;j++)
			{
				string tmp;
				switch ((*d_it)->board[i][j])
				{
				case wKing:		tmp="wK";
								break;
				case wQueen:	tmp="wQ";
								break;
				case wRook:		tmp="wR";
								break;
				case wBishop:	tmp="wB";
								break;
				case wKnight:	tmp="wN";
								break;
				case wPawn:		tmp="wP";
								break;
				case bKing:		tmp="bK";
								break;
				case bQueen:	tmp="bQ";
								break;
				case bRook:		tmp="bR";
								break;
				case bBishop:	tmp="bB";
								break;
				case bKnight:	tmp="bN";
								break;
				case bPawn:		tmp="bP";
								break;
				default:		tmp="--";
								break;
				}
				debug2<<tmp<<" ";
			}
			debug2<<endl;
		}
		debug2<<endl<<(*d_it)->evalValue<<"----------------------"<<endl<<endl;
		d_it++;
	}

	debug2<<"********************************"<<endl;
#endif


	timer.Start(); //global timer used to accumulate all the moves
	Timer moveTimer; //local timer used only to one move
	moveTimer.Reset();
	moveTimer.CountDown(timeAlloc());
	moveTimer.Start();
	
#ifdef OPENBOOK
	if(curConfig->step_count<=5) //may appear in Openbook
	{
		Config config;
		int i;
		for(i=0;i<8;i++)
			for(int j=0;j<8;j++)
				config.board[i][j]=curConfig->board[i][j];
		config.blackNext=curConfig->blackNext;

		for(i=0;i<11;i++)
		{
			if(openbook[i].Search(&config)) //if such an open is found
			{
				Configuration* pConfig=curConfig->makeConfig(config.move);
				(*curConfig)=(*pConfig);
				delete pConfig;
				return config.move;
			}
		}
	}
#endif

	MinimaxTree tree(curConfig, plyLimit, nodeLimit, &moveTimer);
	tree.getResponse(lastTwoConfigs[0]);
	
	moveTimer.Stop();

	moveCounter++;
	timer.Stop();

//	*nodeSearched=tree.getnodeNum();

//	tEnd=clock();
//	*timeExe=((double)tEnd-(double)tStart)*1000/CLOCKS_PER_SEC;

	#ifdef DEBUG2
/*	ofstream debug1("control.txt");
	for(int i=0;i<8;i++)
	{
		for(int j=0;j<8;j++)
		{
			debug1.width(2);
			debug1<<(curConfig->valControl[i][j][2]-curConfig->valControl[i][j][0])<<"  ";
		}
		debug1<<endl;
	}
	debug1<<curConfig->evalValue<<endl;
	debug1<<"----------------------"<<endl<<endl;
*/
	ofstream debug("debug.txt");
	for(int i=0;i<8;i++)
	{
		for(int j=0;j<8;j++)
		{
			string tmp;
			switch (curConfig->board[i][j])
			{
			case wKing:		tmp="wK";
							break;
			case wQueen:	tmp="wQ";
							break;
			case wRook:		tmp="wR";
							break;
			case wBishop:	tmp="wB";
							break;
			case wKnight:	tmp="wN";
							break;
			case wPawn:		tmp="wP";
							break;
			case bKing:		tmp="bK";
							break;
			case bQueen:	tmp="bQ";
							break;
			case bRook:		tmp="bR";
							break;
			case bBishop:	tmp="bB";
							break;
			case bKnight:	tmp="bN";
							break;
			case bPawn:		tmp="bP";
							break;
			default:		tmp="--";
							break;
			}
			debug<<tmp<<" ";
		}
		debug<<endl;
	}
	debug<<curConfig->evalValue<<endl;
	debug<<curConfig->backup_value<<endl;
	debug<<"********************************"<<endl;
	#endif

	//update last two configs
	if(lastTwoConfigs[0] != NULL)
	{
		delete lastTwoConfigs[0];
	}
	lastTwoConfigs[0]=lastTwoConfigs[1];
	lastTwoConfigs[1]=new Configuration(*curConfig);

	if(curConfig->backup_value==MinEvalValue)
	{
		return (MOVE)0xffff;
	}

	return curConfig->move;
}
