#include "BitBoard.h"

unsigned char first_piece[65536];
unsigned char HVControl[256][8];

/* Masks for the FirstPiece algorithm */
//const BITBOARD  FPMask1 = (BITBOARD)TwoFullRanks << 16;
//const BITBOARD  FPMask2 = (BITBOARD)TwoFullRanks << 32;

/* square -> square mappings for a rotation of 90 degrees anti-clockwise */
const int tmpRotateL90[64] = {
	h8,h7,h6,h5,h4,h3,h2,h1,
	g8,g7,g6,g5,g4,g3,g2,g1,
	f8,f7,f6,f5,f4,f3,f2,f1,
	e8,e7,e6,e5,e4,e3,e2,e1,
	d8,d7,d6,d5,d4,d3,d2,d1,
	c8,c7,c6,c5,c4,c3,c2,c1,
	b8,b7,b6,b5,b4,b3,b2,b1,
	a8,a7,a6,a5,a4,a3,a2,a1
};
int RotateL90[64];
/* square -> square mappings for a rotation of 90 degrees clockwise */
const int tmpRotateR90[64] = {
	a1,a2,a3,a4,a5,a6,a7,a8,
	b1,b2,b3,b4,b5,b6,b7,b8,
	c1,c2,c3,c4,c5,c6,c7,c8,
	d1,d2,d3,d4,d5,d6,d7,d8,
	e1,e2,e3,e4,e5,e6,e7,e8,
	f1,f2,f3,f4,f5,f6,f7,f8,
	g1,g2,g3,g4,g5,g6,g7,g8,
	h1,h2,h3,h4,h5,h6,h7,h8
};
int RotateR90[64];
/* This board is used for working out leading diagonals (a1h8 sense).  Effectively
 * it represents the square-square mappings for a rotation of 45 degrees clockwise */
const int tmpRotateR45[64] = {
	a8,
	a7,b8,
	a6,b7,c8,
	a5,b6,c7,d8,
	a4,b5,c6,d7,e8,
	a3,b4,c5,d6,e7,f8,
	a2,b3,c4,d5,e6,f7,g8,
	a1,b2,c3,d4,e5,f6,g7,h8,
	b1,c2,d3,e4,f5,g6,h7,
	c1,d2,e3,f4,g5,h6,
	d1,e2,f3,g4,h5,
	e1,f2,g3,h4,
	f1,g2,h3,
	g1,h2,
	h1
};

int RotateR45[64];
/* This board is used for working out trailing diagonals (a8h1 sense). Effectively
 * it represents the square-square mappings for a rotation of 45 degrees anti-clockwise */
const int tmpRotateL45[64] = {
	h8,
	g8,h7,
	f8,g7,h6,
	e8,f7,g6,h5,
	d8,e7,f6,g5,h4,
	c8,d7,e6,f5,g4,h3,
	b8,c7,d6,e5,f4,g3,h2,
	a8,b7,c6,d5,e4,f3,g2,h1,
	a7,b6,c5,d4,e3,f2,g1,
	a6,b5,c4,d3,e2,f1,
	a5,b4,c3,d2,e1,
	a4,b3,c2,d1,
	a3,b2,c1,
	a2,b1,
	a1   
};

int RotateL45[64];
/* These simply store the length of the diagonal in the required sense */
const int DiagonalLength_a1h8[64] = {
	1,2,3,4,5,6,7,8,
	2,3,4,5,6,7,8,7,
	3,4,5,6,7,8,7,6,
	4,5,6,7,8,7,6,5,
	5,6,7,8,7,6,5,4,
	6,7,8,7,6,5,4,3,
	7,8,7,6,5,4,3,2,
	8,7,6,5,4,3,2,1
};

const int DiagonalLength_a8h1[64] = {
	8,7,6,5,4,3,2,1,
	7,8,7,6,5,4,3,2,
	6,7,8,7,6,5,4,3,
	5,6,7,8,7,6,5,4,
	4,5,6,7,8,7,6,5,
	3,4,5,6,7,8,7,6,
	2,3,4,5,6,7,8,7,
	1,2,3,4,5,6,7,8 
};


//Beuwolf's data stop here

const int DiagonalShift_a1h8[64] = {
	0,1,3,6,10,15,21,28,
	1,3,6,10,15,21,28,36,
	3,6,10,15,21,28,36,43,
	6,10,15,21,28,36,43,49,
	10,15,21,28,36,43,49,54,
	15,21,28,36,43,49,54,58,
	21,28,36,43,49,54,58,61,
	28,36,43,49,54,58,61,63
};

const int DiagonalShift_a8h1[64] = {
	28,21,15,10,6,3,1,0,
	36,28,21,15,10,6,3,1,
	43,36,28,21,15,10,6,3,
	49,43,36,28,21,15,10,6,
	54,49,43,36,28,21,15,10,
	58,54,49,43,36,28,21,15,
	61,58,54,49,43,36,28,21,
	63,61,58,54,49,43,36,28 
};

BITBOARD rotateRight90(BITBOARD initBoard)
{
	BITBOARD result=0;
	for(int i=0;i<64;i++)
	{
		if(initBoard&(Unit<<i))
		{
			result|=Unit<<RotateR90[i];
		}
	}
	return result;
}

BITBOARD rotateLeft90(BITBOARD initBoard)
{
	BITBOARD result=0;
	for(int i=0;i<64;i++)
	{
		if(initBoard&(Unit<<i))
		{
			result|=Unit<<RotateL90[i];
		}
	}
	return result;
}

BITBOARD rotateRight45(BITBOARD initBoard)
{
	BITBOARD result=0;
	for(int i=0;i<64;i++)
	{
		if(initBoard&(Unit<<i))
		{
			result|=Unit<<RotateR45[i];
		}
	}
	return result;
}

BITBOARD reverseRight45(BITBOARD initBoard)
{
	BITBOARD result=0;
	for(int i=0;i<64;i++)
	{
		if(initBoard&(Unit<<i))
		{
			result|=Unit<<tmpRotateR45[i];
		}
	}
	return result;
}

BITBOARD rotateLeft45(BITBOARD initBoard)
{
	BITBOARD result=0;
	for(int i=0;i<64;i++)
	{
		if(initBoard&(Unit<<i))
		{
			result|=Unit<<RotateL45[i];
		}
	}
	return result;
}

BITBOARD reverseLeft45(BITBOARD initBoard)
{
	BITBOARD result=0;
	for(int i=0;i<64;i++)
	{
		if(initBoard&(Unit<<i))
		{
			result|=Unit<<tmpRotateL45[i];
		}
	}
	return result;
}

//initialize should be called first in the controller function
void makeRotateTable()
{
	for(int i=0;i<64;i++)
		for(int j=0;j<64;j++)
		{
			if((int)tmpRotateR90[j]==i)
				RotateR90[i]=j;
			if((int)tmpRotateL90[j]==i)
				RotateL90[i]=j;
			if((int)tmpRotateR45[j]==i)
				RotateR45[i]=j;
			if((int)tmpRotateL45[j]==i)
				RotateL45[i]=j;
		}
}
void makeFirstPieceTable()
{
	for (int i=0;i<65536;i++) 
	{
     /* First piece in a two rank chunk */
		for (int j=0;j<16;j++) 
		{
			if (i&(1<<j)) 
			{
				first_piece[i] = (unsigned char) j; 
				break;
			}
		}
    }
}

void makeHVControlTable()
{
	int unit=1;
	int head1;
	int head2;
	int head3;
	int tmp;
	for(int i=0;i<256;i++)
	{
		tmp=unit;
		head1=-1;
		head2=head3=0;
		while(head3<8)
		{
			if((tmp&i)==0)
			{
				HVControl[i][head2]=0;
				head3++;
				tmp=tmp<<1;
				continue;
			}
			if(head3>head1&&head1!=-1)
			{
				HVControl[i][head2]=(0xFF>>(7-head3))&(0xFF<<head1)&(~(unit<<head2));
			}
			head1=head2;
			head2=head3;
			head3++;
			tmp=tmp<<1;
		}
		if(head1>=0)
		{
			HVControl[i][head2]=0xFF&(0xFF<<head1)&(~(unit<<head2));
		}
	}
}

int pickFirst(BITBOARD board)
{
	if (board&0xFFFF)
	{
		return first_piece[board&0xFFFF];
	}

	board=board>>16;
	if (board&0xFFFF) 
	{
		return first_piece[board&0xFFFF] + 16;
	}

	board=board>>16;
	if (board&0xFFFF) 
	{
		return first_piece[board&0xFFFF] + 32;
	}

	board=board>>16;
	if (board&0xFFFF) 
	{
		return first_piece[board&0xFFFF] + 48;
	}

	return -1;
}

BITBOARD getHControl(BITBOARD board, int pos)
{
	int rank;
	int file;
	rank=pos>>3;
	file=pos-(rank<<3);
	
	int p1=(int)((board>>(rank<<3))&255);
	BITBOARD hControl=(BITBOARD)(HVControl[p1][file])<<(rank<<3);

	return hControl;
}

BITBOARD getVControl(BITBOARD board, int pos)
{
	BITBOARD rotateBoard=rotateRight90(board);

	int rank;
	int file;
	rank=pos>>3;
	file=pos-(rank<<3);
	
	int rotatePos=file*8+(7-rank);

	BITBOARD rotateControl=getHControl(rotateBoard,rotatePos);

	return rotateLeft90(rotateControl);
}
	
BITBOARD getHVControl(BITBOARD board, int pos)
{
	return getHControl(board,pos) | getVControl(board,pos);
}

BITBOARD getA1H8Control(BITBOARD board, int pos)	
{

	int len=DiagonalLength_a1h8[pos];
	int shift=DiagonalShift_a1h8[pos];

	BITBOARD rotateBoard=rotateRight45(board);
	rotateBoard=rotateBoard>>shift;
	int p1=(int)(rotateBoard&(0xFF>>(8-len)));
	
	int rank;
	int file;
	rank=pos>>3;
	file=pos-(rank<<3);

	int pos1=(file<=(7-rank)?file:(7-rank)); //min(file, 7-rank)
		
	BITBOARD xControl=((BITBOARD)(HVControl[p1][pos1]&(0xFF>>(8-len))))<<shift;

	xControl=reverseRight45(xControl);

	return xControl;
}

BITBOARD getA8H1Control(BITBOARD board, int pos)	
{

	int len=DiagonalLength_a8h1[pos];
	int shift=DiagonalShift_a8h1[pos];

	BITBOARD rotateBoard=rotateLeft45(board);
	rotateBoard=rotateBoard>>shift;
	int p1=(int)(rotateBoard&(0xFF>>(8-len)));
	
	int rank;
	int file;
	rank=pos>>3;
	file=pos-(rank<<3);

	int pos1=(rank<=file?rank:file);
	
	BITBOARD xControl=(BITBOARD)(HVControl[p1][pos1]&(0xFF>>(8-len)))<<shift;

	xControl=reverseLeft45(xControl);

	return xControl;
}

BITBOARD getXControl(BITBOARD board, int pos)
{
	return getA1H8Control(board,pos) | getA8H1Control(board,pos);
}

void initBitBoard()
{
	makeRotateTable();
	makeFirstPieceTable();
	makeHVControlTable();
}

