// Configuration.cpp: implementation of the Configuration class.
//
//////////////////////////////////////////////////////////////////////
#include <list>

#include "Configuration.h"
#include "BitBoard.h"

#ifdef DEBUG
#include <fstream>
#include <string>
using namespace std;
#endif

//////////////////////////////////////////////////////////////////////
// Construction FOR STRUCT Flags
//////////////////////////////////////////////////////////////////////
Flags::Flags()
{
	leftBlackRookMoved=false; //if left black rook moved, used for castling
	rightBlackRookMoved=false; //if left black rook moved, used for castling
	leftWhiteRookMoved=false; //if left black rook moved, used for castling
	rightWhiteRookMoved=false; //if left black rook moved, used for castling
	blackKingMoved=false;
	whiteKingMoved=false;
	blackCastled=false;
	whiteCastled=false;

	enPassant=false;
	promotion=0; //the black and white will be distingished in makeConfig
}

Flags::Flags(Flags& flags)
{
	leftBlackRookMoved=flags.leftBlackRookMoved; //if left black rook moved, used for castling
	rightBlackRookMoved=flags.rightBlackRookMoved; //if left black rook moved, used for castling
	leftWhiteRookMoved=flags.leftWhiteRookMoved; //if left black rook moved, used for castling
	rightWhiteRookMoved=flags.rightWhiteRookMoved; //if left black rook moved, used for castling
	blackKingMoved=flags.blackKingMoved;
	whiteKingMoved=flags.whiteKingMoved;
	blackCastled=flags.blackCastled;
	whiteCastled=flags.whiteCastled;

	enPassant=flags.enPassant;
	promotion=flags.promotion;
}

Flags& Flags::operator=(Flags& flags)
{
	leftBlackRookMoved=flags.leftBlackRookMoved; //if left black rook moved, used for castling
	rightBlackRookMoved=flags.rightBlackRookMoved; //if left black rook moved, used for castling
	leftWhiteRookMoved=flags.leftWhiteRookMoved; //if left black rook moved, used for castling
	rightWhiteRookMoved=flags.rightWhiteRookMoved; //if left black rook moved, used for castling
	blackKingMoved=flags.blackKingMoved;
	whiteKingMoved=flags.whiteKingMoved;
	blackCastled=flags.blackCastled;
	whiteCastled=flags.whiteCastled;

	enPassant=flags.enPassant;
	promotion=flags.promotion;

	return *this;
}

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

/*
Configuration::Configuration(MoveGen* pMoveGen)
{
	for(int i=0;i<8;i++)
		for(int j=0;j<8;j++)
		{
			board[i][j]=InitBoard[i*8+j];
			for(int k=0;k<4;k++)
			{
				valControl[i][j][k]=MaxValue;
			}
		}

	amIBlack=true; //true if the engine stands for black
	blackNext=true; //true means next is black
	leftBlackRookMoved=false; //if left black rook moved, used for castling
	rightBlackRookMoved=false; //if left black rook moved, used for castling
	leftWhiteRookMoved=false; //if left black rook moved, used for castling
	rightWhiteRookMoved=false; //if left black rook moved, used for castling
	blackKingMoved=false;
	whiteKingMoved=false;
	blackCastled=false;
	whiteCastled=false;

//	m_pEvaluator=pEvaluator;
	m_pMoveGen=pMoveGen;
}
*/

Configuration::Configuration(Evaluator* pEvaluator, MoveGen* pMoveGen): flags()
{
	amIBlack=true; //true if the engine stands for black
	blackNext=true; //true means next is black

	for(int i=0;i<8;i++)
		for(int j=0;j<8;j++)
		{
			board[i][j]=InitBoard[i*8+j];
			valControl[i][j][0]=valControl[i][j][2]=0;
			valControl[i][j][1]=valControl[i][j][3]=MaxValue;
		}

	evalValue=0;
	parentEval=0;
	move=0;

	step_count=0;
	backup_value=MinEvalValue;

	m_pEvaluator=pEvaluator;
	m_pMoveGen=pMoveGen;

	cmpBitBoard(); //computer some bitboard needed in future
	cmpControl(); //computer the control matrix
}

Configuration::Configuration(Evaluator* pEvaluator, Evaluator* pEvaluator2,MoveGen* pMoveGen, MoveGen* pMoveGen2): flags()
{
	amIBlack=true; //true if the engine stands for black
	blackNext=true; //true means next is black

	for(int i=0;i<8;i++)
		for(int j=0;j<8;j++)
		{
			board[i][j]=InitBoard[i*8+j];
			valControl[i][j][0]=valControl[i][j][2]=0;
			valControl[i][j][1]=valControl[i][j][3]=MaxValue;
		}

	evalValue=0;
	parentEval=0;
	move=0;

	step_count=0;
	backup_value=MinEvalValue;

	m_pEvaluator=pEvaluator;
	m_pEvaluator2=pEvaluator2;
	m_pMoveGen=pMoveGen;
	m_pMoveGen2=pMoveGen2;

	cmpBitBoard(); //computer some bitboard needed in future
	cmpControl(); //computer the control matrix
}

Configuration::Configuration(Configuration* config) : flags(config->flags)
{
	amIBlack=config->amIBlack; //true if the engine stands for black
	blackNext=config->blackNext; //true means next is black
	
	for(int i=0;i<8;i++)
		for(int j=0;j<8;j++)
		{
			board[i][j]=config->board[i][j];
			for(int k=0;k<4;k++)
			{
				valControl[i][j][0]=valControl[i][j][2]=0;
				valControl[i][j][1]=valControl[i][j][3]=MaxValue;			
			}
		}

	evalValue=config->evalValue;
	parentEval=config->parentEval;
	move=config->move;

	step_count=config->step_count;
	backup_value=config->backup_value;

	m_pEvaluator=config->m_pEvaluator;
	m_pEvaluator2=config->m_pEvaluator2;
	m_pMoveGen=config->m_pMoveGen;
	m_pMoveGen2=config->m_pMoveGen2;

	cmpBitBoard(); //computer some bitboard needed in future
	cmpControl(); //computer the control matrix
}

Configuration::~Configuration()
{

}

//compute the bidboards
void Configuration::cmpBitBoard()
{
	wK=wQ=wB=wN=wR=wP=0;
	bK=bQ=bB=bN=bR=bP=0;
	BITBOARD unit=1;
	for(int i=0;i<8;i++)
		for(int j=0;j<8;j++)
		{
			switch(board[i][j])
			{
			case wKing:
			{
				wK|=(unit<<((i<<3)+j));
				break;
			}
			case wQueen:
			{
				wQ|=(unit<<((i<<3)+j));
				break;
			}
			case wBishop:
			{
				wB|=(unit<<((i<<3)+j));
				break;
			}
			case wKnight:
			{
				wN|=(unit<<((i<<3)+j));
				break;
			}
			case wRook:
			{
				wR|=(unit<<((i<<3)+j));
				break;
			}
			case wPawn:
			{
				wP|=(unit<<((i<<3)+j));
				break;
			}
			case bKing:
			{
				bK|=(unit<<((i<<3)+j));
				break;
			}
			case bQueen:
			{
				bQ|=(unit<<((i<<3)+j));
				break;
			}
			case bBishop:
			{
				bB|=(unit<<((i<<3)+j));
				break;
			}
			case bKnight:
			{
				bN|=(unit<<((i<<3)+j));
				break;
			}
			case bRook:
			{
				bR|=(unit<<((i<<3)+j));
				break;
			}
			case bPawn:
			{
				bP|=(unit<<((i<<3)+j));
				break;
			}
			default:
				{}
			}
		}
	
	//computer other bitboards
	wOccupied=wK|wQ|wB|wN|wR|wP;
	bOccupied=bK|bQ|bB|bN|bR|bP;
	allOccupied=wOccupied|bOccupied;
	allR90=rotateRight90(allOccupied);
	allR45=rotateRight45(allOccupied);
	allL45=rotateLeft45(allOccupied);
}

void Configuration::cmpControl()
{
/*	BITBOARD bPCtrl[2];
	BITBOARD bNCtrl[8];
	BITBOARD bBCtrl[2];
	BITBOARD bRCtrl[2];
	BITBOARD bQCtrl[1];
	BITBOARD bKCtrl[1];
	BITBOARD wPCtrl[2];
	BITBOARD wNCtrl[2];
	BITBOARD wBCtrl[2];
	BITBOARD wRCtrl[2];
	BITBOARD wQCtrl[1];
	BITBOARD wKCtrl[1];
*/
	BITBOARD tmpPCtrl[2];
	BITBOARD tmpNCtrl[8];
	BITBOARD tmpBCtrl[2];
	BITBOARD tmpRCtrl[2];
	BITBOARD tmpQCtrl[1];
	BITBOARD tmpKCtrl[1];

	int tmpIndex1;
	int tmpIndex2;
	int tmpPos;
	int rank;
	int file;

//	BITBOARD tmpCtrl;
	BITBOARD tmpP;
	BITBOARD tmpN;
	BITBOARD tmpB;
	BITBOARD tmpR;
	BITBOARD tmpQ;
	BITBOARD tmpK;


	BITBOARD sideOccupied;
	BITBOARD oppoOccupied;

	int i;

	for(i=0;i<8;i++) //clear first
		for(int j=0;j<8;j++)
		{
			valControl[i][j][0]=valControl[i][j][2]=0;
			valControl[i][j][1]=valControl[i][j][3]=MaxValue;
		}


	for(int k=0;k<2;k++) //k=0 black, k=1 white
	{
		if(k==0)
		{
			tmpP=bP;
			tmpN=bN;
			tmpB=bB;
			tmpR=bR;
			tmpQ=bQ;
			tmpK=bK;
			
			tmpIndex1=0;
			tmpIndex2=1;
/*			tmpPCtrl=bPCtrl;
			tmpNCtrl=bNCtrl;
			tmpBCtrl=bBCtrl;
			tmpRCtrl=bRCtrl;
			tmpQCtrl=bQCtrl;
			tmpKCtrl=bKCtrl;
*/
			sideOccupied=bOccupied;
			oppoOccupied=wOccupied;
		}
		else
		{
			tmpP=wP;
			tmpN=wN;
			tmpB=wB;
			tmpR=wR;
			tmpQ=wQ;
			tmpK=wK;

			tmpIndex1=2;
			tmpIndex2=3;
			sideOccupied=wOccupied;
			oppoOccupied=bOccupied;
/*			tmpPCtrl=wPCtrl;
			tmpNCtrl=wNCtrl;
			tmpBCtrl=wBCtrl;
			tmpRCtrl=wRCtrl;
			tmpQCtrl=wQCtrl;
			tmpKCtrl=wKCtrl;
*/
		}
/*		
		#ifdef DEBUG
			ofstream debug("control.txt");
			for(i=0;i<8;i++)
			{
				for(int j=0;j<8;j++)
				{
					debug.width(2);
					debug<<(valControl[i][j][2]-valControl[i][j][0])<<"  ";
				}
				debug<<endl;
			}
			debug<<"----------------------"<<endl<<endl;
		#endif
*/
		//Pawn capture
		if(k==0) //black
		{
			tmpPCtrl[0]=((bP&~C1)<<7);
			tmpPCtrl[1]=((bP&~C8)<<9);
			for(i=0;i<2;i++)
			{
				while(tmpPCtrl[i]!=0)
				{
					tmpPos=pickFirst(tmpPCtrl[i]);
					rank=tmpPos>>3;
					file=tmpPos-(rank<<3);
					valControl[rank][file][tmpIndex1]+=1;

					if (valControl[rank][file][tmpIndex2]>PawnValue)
					{
						valControl[rank][file][tmpIndex2]=PawnValue;
					}
					tmpPCtrl[i]&=~(Unit<<tmpPos);
				}
			}
		}
		else
		{
			tmpPCtrl[0]=((wP&~C8)>>7);
			tmpPCtrl[1]=((wP&~C1)>>9);
			for(i=0;i<2;i++)
			{
				while(tmpPCtrl[i]!=0)
				{
					tmpPos=pickFirst(tmpPCtrl[i]);
					rank=tmpPos>>3;
					file=tmpPos-(rank<<3);
					valControl[rank][file][tmpIndex1]+=1;
					if(PawnValue<valControl[rank][file][tmpIndex2])
					{
						valControl[rank][file][tmpIndex2]=PawnValue;
					}
					tmpPCtrl[i]&=~(Unit<<tmpPos);
				}
			}
		}
/*
		#ifdef DEBUG
			//ofstream debug("control.txt");
			for(i=0;i<8;i++)
			{
				for(int j=0;j<8;j++)
				{
					debug.width(2);
					debug<<(valControl[i][j][2]-valControl[i][j][0])<<"  ";
				}
				debug<<endl;
			}
			debug<<"----------------------"<<endl<<endl;
		#endif
*/
		//Knight capture
		tmpNCtrl[0]=((tmpN&~C7&~C8)>>6);
		tmpNCtrl[1]=((tmpN&~C7&~C8)<<10);
		tmpNCtrl[2]=((tmpN&~C1&~C2)>>10);
		tmpNCtrl[3]=((tmpN&~C1&~C2)<<6);
		tmpNCtrl[4]=((tmpN&~C1)>>17);
		tmpNCtrl[5]=((tmpN&~C8)>>15);
		tmpNCtrl[6]=((tmpN&~C8)<<17);
		tmpNCtrl[7]=((tmpN&~C1)<<15);
		for(i=0;i<8;i++)
		{
			while(tmpNCtrl[i]!=0)
			{
				tmpPos=pickFirst(tmpNCtrl[i]);
				rank=tmpPos>>3;
				file=tmpPos-(rank<<3);
				valControl[rank][file][tmpIndex1]+=1;
				if(KnightValue<valControl[rank][file][tmpIndex2])
				{
					valControl[rank][file][tmpIndex2]=KnightValue;
				}
				tmpNCtrl[i]&=~(Unit<<tmpPos);
			}
		}
/*
		#ifdef DEBUG
			//ofstream debug("control.txt");
			for(i=0;i<8;i++)
			{
				for(int j=0;j<8;j++)
				{
					debug.width(2);
					debug<<(valControl[i][j][2]-valControl[i][j][0])<<"  ";
				}
				debug<<endl;
			}
			debug<<"----------------------"<<endl<<endl;
		#endif
*/
		//King
		if(tmpK==0)
		{
		}
		else if((tmpK&~C1)==0)
		{
			tmpKCtrl[0]=(tmpK<<1)|(tmpK<<8)|(tmpK<<9)|(tmpK>>7)|(tmpK>>8);
		}
		else if((tmpK&~C8)==0)
		{
			tmpKCtrl[0]=(tmpK<<7)|(tmpK<<8)|(tmpK>>1)|(tmpK>>8)|(tmpK>>9);
		}
		else
		{
			tmpKCtrl[0]=(tmpK<<1)|(tmpK<<7)|(tmpK<<8)|(tmpK<<9)|(tmpK>>1)|(tmpK>>7)|(tmpK>>8)|(tmpK>>9);
		}
		for(i=0;i<1;i++)
		{
			while(tmpKCtrl[i]!=0)
			{
				tmpPos=pickFirst(tmpKCtrl[i]);
				rank=tmpPos>>3;
				file=tmpPos-(rank<<3);
				valControl[rank][file][tmpIndex1]+=1;
				if(KingValue<valControl[rank][file][tmpIndex2])
				{
					valControl[rank][file][tmpIndex2]=KingValue;
				}
				tmpKCtrl[i]&=~(Unit<<tmpPos);
			}
		}
/*
		#ifdef DEBUG
			//ofstream debug("control.txt");
			for(i=0;i<8;i++)
			{
				for(int j=0;j<8;j++)
				{
					debug.width(2);
					debug<<(valControl[i][j][2]-valControl[i][j][0])<<"  ";
				}
				debug<<endl;
			}
			debug<<"----------------------"<<endl<<endl;
		#endif
*/
		//Rook
		int pos;
		int i=0;
		while(tmpR!=0) //for each rook
		{
			pos=pickFirst(tmpR);
			//get horizonal and vertical control
			tmpRCtrl[i]=getHVControl(allOccupied,pos);
			while(tmpRCtrl[i]!=0)
			{
				tmpPos=pickFirst(tmpRCtrl[i]);
				rank=tmpPos>>3;
				file=tmpPos-(rank<<3);
				valControl[rank][file][tmpIndex1]+=1;
				if(RookValue<valControl[rank][file][tmpIndex2])
				{
					valControl[rank][file][tmpIndex2]=RookValue;
				}
				tmpRCtrl[i]&=~(Unit<<tmpPos);
			}
			tmpR&=~(Unit<<pos);
			if(++i>1) break;
		}
/*		
		#ifdef DEBUG
			//ofstream debug("control.txt");
			for(i=0;i<8;i++)
			{
				for(int j=0;j<8;j++)
				{
					debug.width(2);
					debug<<(valControl[i][j][2]-valControl[i][j][0])<<"  ";
				}
				debug<<endl;
			}
			debug<<"----------------------"<<endl<<endl;
		#endif
*/
		//Bishop
		i=0;
		while(tmpB!=0)
		{
			pos=pickFirst(tmpB);
			//get diagonal control
			tmpBCtrl[i]=getXControl(allOccupied,pos);
			while(tmpBCtrl[i]!=0)
			{
				tmpPos=pickFirst(tmpBCtrl[i]);
				rank=tmpPos>>3;
				file=tmpPos-(rank<<3);
				valControl[rank][file][tmpIndex1]+=1;
				if(BishopValue<valControl[rank][file][tmpIndex2])
				{
					valControl[rank][file][tmpIndex2]=BishopValue;
				}
				tmpBCtrl[i]&=~(Unit<<tmpPos);
			}
			tmpB&=~(Unit<<pos);
			if(++i>1) break;
		}
/*
		#ifdef DEBUG
			//ofstream debug("control.txt");
			for(i=0;i<8;i++)
			{
				for(int j=0;j<8;j++)
				{
					debug.width(2);
					debug<<(valControl[i][j][2]-valControl[i][j][0])<<"  ";
				}
				debug<<endl;
			}
			debug<<"----------------------"<<endl<<endl;
		#endif
*/
		//Queen
		i=0;
		while(tmpQ!=0)
		{
			pos=pickFirst(tmpQ);
			//get horizonal and vertical control
			tmpQCtrl[i]=getHVControl(allOccupied,pos);
			//get diagonal control
			tmpQCtrl[i]=(getXControl(allOccupied,pos)) | tmpQCtrl[i];
			while(tmpQCtrl[i]!=0)
			{
				tmpPos=pickFirst(tmpQCtrl[i]);
				rank=tmpPos>>3;
				file=tmpPos-(rank<<3);
				valControl[rank][file][tmpIndex1]+=1;
				if(QueenValue<valControl[rank][file][tmpIndex2])
				{
					valControl[rank][file][tmpIndex2]=QueenValue;
				}
				tmpQCtrl[i]&=~(Unit<<tmpPos);
			}
			tmpQ&=~(Unit<<pos);
		}
/*
		#ifdef DEBUG
			//ofstream debug("control.txt");
			for(i=0;i<8;i++)
			{
				for(int j=0;j<8;j++)
				{
					debug.width(2);
					debug<<(valControl[i][j][2]-valControl[i][j][0])<<"  ";
				}
				debug<<endl;
			}
			debug<<"----------------------"<<endl<<endl;
		#endif
*/
	}
}
	
int Configuration::evaluate()
{
	return evalValue=m_pEvaluator->evaluate(this);
}

int Configuration::evaluate2()
{
	return evalValue=m_pEvaluator2->evaluate(this);
}

ConfigList* Configuration::successors()
{
	return m_pMoveGen->successors(this);
}

ConfigList* Configuration::successors2()
{
	return m_pMoveGen2->successors(this);
	//return m_pMoveGen->successors(this);
}

Configuration* Configuration::makeConfig(MOVE move)
{
	Configuration* newConfig=new Configuration(this);
	newConfig->blackNext=!blackNext;
	newConfig->parentEval=evalValue;
	newConfig->move=move;
	newConfig->step_count=step_count+1;
	newConfig->flags.enPassant=0;
	newConfig->flags.promotion=0;

	int from;
	int to;
	to=move&0xFF;
	from=move>>8;
	int rank1;
	int file1;
	int rank2;
	int file2;
	rank1=from>>3;
	file1=from-(rank1<<3);
	rank2=to>>3;
	file2=to-(rank2<<3);
	
	newConfig->board[rank2][file2]=newConfig->board[rank1][file1];
	newConfig->board[rank1][file1]=0;
	
	//castling
	if(move==0x0406&&board[rank1][file1]==bKing)
	{
		newConfig->board[0][7]=0;
		newConfig->board[0][5]=bRook;
		newConfig->flags.blackCastled=true;
	}
	if(move==0x0402&&board[rank1][file1]==bKing)
	{
		newConfig->board[0][0]=0;
		newConfig->board[0][3]=bRook;
		newConfig->flags.blackCastled=true;
	}
	if(move==0x3c3e&&board[rank1][file1]==wKing)
	{
		newConfig->board[7][7]=0;
		newConfig->board[7][5]=wRook;
		newConfig->flags.whiteCastled=true;
	}
	if(move==0x3c3a&&board[rank1][file1]==wKing)
	{
		newConfig->board[7][0]=0;
		newConfig->board[7][3]=wRook;
		newConfig->flags.whiteCastled=true;
	}
	//update some flags
	if(0x00==from||0x00==to) 
	{
		newConfig->flags.leftBlackRookMoved=true;
	}
	if(0x07==from||0x07==to) 
	{
		newConfig->flags.rightBlackRookMoved=true;
	}
	if(0x38==from||0x38==to) 
	{
		newConfig->flags.leftWhiteRookMoved=true;
	}
	if(0x3F==from||0x3F==to) 
	{
		newConfig->flags.rightWhiteRookMoved=true;
	}
	if(0x04==from||0x04==to) 
	{
		newConfig->flags.blackKingMoved=true;
	}
	if(0x3C==from||0x3C==to) 
	{
		newConfig->flags.whiteKingMoved=true;
	}

	//en passant & promotion
	if((newConfig->board[rank2][file2]&0x02)!=0) //pawn
	{
		//en passant
		if(2==(rank2-rank1)||-2==(rank2-rank1)) //2 push pawn, may be eaten as passant
		{
			newConfig->flags.enPassant= Unit<<to; //record the pos
		}
		if(1==(file2-file1)||-1==(file2-file1)) //pawn capture
		{
			if(0==board[rank2][file2]) //the square is original blank, then have to capture en passant
			{
				newConfig->board[rank1][file2]=0; //cross out en passant
			}
		}

		//promotion
		if(0==rank2) //white pawn promotion
		{
			if(amIBlack) //opponent do promotion
			{
				if(flags.promotion != 0)
				{
					newConfig->board[rank2][file2]=flags.promotion;
				}
			}
			else //I do promotion, use queen
			{
				newConfig->board[rank2][file2]=wQueen;
				newConfig->flags.promotion=wQueen;
			}
		}
		if(7==rank2) //black pawn promotion
		{
			if(!amIBlack) //opponent do promotion
			{
				if(flags.promotion != 0)
				{
					newConfig->board[rank2][file2]=flags.promotion | 0x80;
				}
			}
			else //I do promotion, use queen
			{
				newConfig->board[rank2][file2]=bQueen;
				newConfig->flags.promotion=wQueen;
			}
		}
	}

	newConfig->cmpBitBoard(); //computer some bitboard needed in future
	newConfig->cmpControl(); //computer the control matrix

	return newConfig;
}
/*
Configuration* Configuration::makeConfig2(MOVE move)
{
	Configuration* newConfig=new Configuration(this);
	newConfig->blackNext=!blackNext;
	newConfig->parentEval=evalValue;
	newConfig->move=move;
	newConfig->step_count=step_count+1;
	newConfig->flags.enPassant=0;
	newConfig->flags.promotion=0;

	int from;
	int to;
	to=move&0xFF;
	from=move>>8;
	int rank1;
	int file1;
	int rank2;
	int file2;
	rank1=from>>3;
	file1=from-(rank1<<3);
	rank2=to>>3;
	file2=to-(rank2<<3);
	
	newConfig->board[rank2][file2]=newConfig->board[rank1][file1];
	newConfig->board[rank1][file1]=0;
	
	//en passant & promotion
	if((newConfig->board[rank2][file2]&0x02)!=0) //pawn
	{
		//en passant
		if(2==(rank2-rank1)||-2==(rank2-rank1)) //2 push pawn, may be eaten as passant
		{
			newConfig->flags.enPassant= Unit<<to; //record the pos
		}
		if(1==(file2-file1)||-1==(file2-file1)) //pawn capture
		{
			if(0==board[rank2][file2]) //the square is original blank, then have to capture en passant
			{
				newConfig->board[rank1][file2]=0; //cross out en passant
			}
		}

		//promotion
		if(0==rank2) //white pawn promotion
		{
			if(amIBlack) //opponent do promotion
			{
				if(flags.promotion != 0)
				{
					newConfig->board[rank2][file2]=flags.promotion;
				}
			}
			else //I do promotion, use queen
			{
				newConfig->board[rank2][file2]=wQueen;
				newConfig->flags.promotion=wQueen;
			}
		}
		if(7==rank2) //black pawn promotion
		{
			if(!amIBlack) //opponent do promotion
			{
				if(flags.promotion != 0)
				{
					newConfig->board[rank2][file2]=flags.promotion | 0x80;
				}
			}
			else //I do promotion, use queen
			{
				newConfig->board[rank2][file2]=bQueen;
				newConfig->flags.promotion=wQueen;
			}
		}
	}

	newConfig->cmpBitBoard(); //computer some bitboard needed in future

	//don't compute control for secondary search
	//newConfig->cmpControl(); 

	return newConfig;
}
*/
Configuration* Configuration::makeConfig2(MOVE move)
{
	Configuration* newConfig=new Configuration(this);
	newConfig->blackNext=!blackNext;
	newConfig->parentEval=evalValue;
	newConfig->move=move;
	newConfig->step_count=step_count+1;
	newConfig->flags.enPassant=0;
	newConfig->flags.promotion=0;

	int from;
	int to;
	to=move&0xFF;
	from=move>>8;
	int rank1;
	int file1;
	int rank2;
	int file2;
	rank1=from>>3;
	file1=from-(rank1<<3);
	rank2=to>>3;
	file2=to-(rank2<<3);
	
	newConfig->board[rank2][file2]=newConfig->board[rank1][file1];
	newConfig->board[rank1][file1]=0;
	
	//castling
	if(move==0x0406&&board[rank1][file1]==bKing)
	{
		newConfig->board[0][7]=0;
		newConfig->board[0][5]=bRook;
		newConfig->flags.blackCastled=true;
	}
	if(move==0x0402&&board[rank1][file1]==bKing)
	{
		newConfig->board[0][0]=0;
		newConfig->board[0][3]=bRook;
		newConfig->flags.blackCastled=true;
	}
	if(move==0x3c3e&&board[rank1][file1]==wKing)
	{
		newConfig->board[7][7]=0;
		newConfig->board[7][5]=wRook;
		newConfig->flags.whiteCastled=true;
	}
	if(move==0x3c3a&&board[rank1][file1]==wKing)
	{
		newConfig->board[7][0]=0;
		newConfig->board[7][3]=wRook;
		newConfig->flags.whiteCastled=true;
	}
	//update some flags
	if(0x00==from||0x00==to) 
	{
		newConfig->flags.leftBlackRookMoved=true;
	}
	if(0x07==from||0x07==to) 
	{
		newConfig->flags.rightBlackRookMoved=true;
	}
	if(0x38==from||0x38==to) 
	{
		newConfig->flags.leftWhiteRookMoved=true;
	}
	if(0x3F==from||0x3F==to) 
	{
		newConfig->flags.rightWhiteRookMoved=true;
	}
	if(0x04==from||0x04==to) 
	{
		newConfig->flags.blackKingMoved=true;
	}
	if(0x3C==from||0x3C==to) 
	{
		newConfig->flags.whiteKingMoved=true;
	}

	//en passant & promotion
	if((newConfig->board[rank2][file2]&0x02)!=0) //pawn
	{
		//en passant
		if(2==(rank2-rank1)||-2==(rank2-rank1)) //2 push pawn, may be eaten as passant
		{
			newConfig->flags.enPassant= Unit<<to; //record the pos
		}
		if(1==(file2-file1)||-1==(file2-file1)) //pawn capture
		{
			if(0==board[rank2][file2]) //the square is original blank, then have to capture en passant
			{
				newConfig->board[rank1][file2]=0; //cross out en passant
			}
		}

		//promotion
		if(0==rank2) //white pawn promotion
		{
			if(amIBlack) //opponent do promotion
			{
				if(flags.promotion != 0)
				{
					newConfig->board[rank2][file2]=flags.promotion;
				}
			}
			else //I do promotion, use queen
			{
				newConfig->board[rank2][file2]=wQueen;
				newConfig->flags.promotion=wQueen;
			}
		}
		if(7==rank2) //black pawn promotion
		{
			if(!amIBlack) //opponent do promotion
			{
				if(flags.promotion != 0)
				{
					newConfig->board[rank2][file2]=flags.promotion | 0x80;
				}
			}
			else //I do promotion, use queen
			{
				newConfig->board[rank2][file2]=bQueen;
				newConfig->flags.promotion=wQueen;
			}
		}
	}
	newConfig->cmpBitBoard(); //computer some bitboard needed in future

	//don't compute control for secondary search
	//newConfig->cmpControl(); 

	return newConfig;
}

void Configuration::setEvaluator(Evaluator* evaluator)
{
	m_pEvaluator=evaluator;
}

bool Configuration::isEndGame()
{
	int BM_value=0;
	int WM_value=0;
	
	for (int i=0;i<8;i++)
		for (int j=0;j<8;j++)
			switch (board[i][j])
			{
			case wQueen:	WM_value+=9;
							break;
			case wRook:		WM_value+=5;	
							break;
			case wBishop:	WM_value+=3;	
							break;
			case wKnight:	WM_value+=3;	
							break;
			case wPawn:		WM_value+=1;	
							break;
			case bQueen:	BM_value+=9;	
							break;
			case bRook:		BM_value+=5;	
							break;
			case bBishop:	BM_value+=3;
							break;
			case bKnight:	BM_value+=3;
							break;
			case bPawn:		BM_value+=1;
							break;
			default:		break;
			}
	
	if ((amIBlack) && (WM_value<16))
		return true;
	if ((!amIBlack) && (BM_value<16))
		return true;
	
	return false;
}
/*
int* Configuration::operator[](int index)
{
	if(index<0 || index>=3)
	{
		return NULL;
	}
	else
	{
		return data[index];
	}
}
*/
Configuration& Configuration::operator=(Configuration& config)
{
	amIBlack=config.amIBlack; //true if the engine stands for black
	blackNext=config.blackNext; //true means next is black
	flags=config.flags;

	for(int i=0;i<8;i++)
		for(int j=0;j<8;j++)
		{
			board[i][j]=config.board[i][j];
			for(int k=0;k<4;k++)
			{
				valControl[i][j][0]=config.valControl[i][j][0];
				valControl[i][j][1]=config.valControl[i][j][1];
				valControl[i][j][2]=config.valControl[i][j][2];
				valControl[i][j][3]=config.valControl[i][j][3];			
			}
		}

		
	evalValue=config.evalValue;
	parentEval=config.parentEval;
	move=config.move;

	step_count=config.step_count;
	backup_value=config.backup_value;

	m_pEvaluator=config.m_pEvaluator;
	m_pEvaluator2=config.m_pEvaluator2;
	m_pMoveGen=config.m_pMoveGen;
	m_pMoveGen2=config.m_pMoveGen2;

	return *this;
}

bool Configuration::equal(Configuration& config)
{
	for(int i=0;i<8;i++)
		for(int j=0;j<8;j++)
		{
			if(board[i][j] != config.board[i][j])
				return false;
		}
	return true;
}
