// Configuration.h: interface for the Configuration class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_CONFIGURATION_H__A76BF06E_1366_4226_B9B2_CAF884669CD9__INCLUDED_)
#define AFX_CONFIGURATION_H__A76BF06E_1366_4226_B9B2_CAF884669CD9__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000


#include "chessRef.h"

class Configuration;

#include <list>
typedef std::list<Configuration*> ConfigList;
typedef std::list<MOVE> MoveList;

#include "Evaluator.h"
#include "MoveGen.h"

struct Flags
{
	Flags();
	Flags(Flags& flags);
	Flags& Flags::operator=(Flags& flags);

	bool leftBlackRookMoved; //if left black rook moved, used for castling
	bool rightBlackRookMoved; //if left black rook moved, used for castling
	bool leftWhiteRookMoved; //if left black rook moved, used for castling
	bool rightWhiteRookMoved; //if left black rook moved, used for castling
	bool blackKingMoved;
	bool whiteKingMoved;
	bool blackCastled;
	bool whiteCastled;

	BITBOARD enPassant; //indicate the pos of En-Passant, 0 means no

	PIECE promotion; //indicate the piece after promotion, 0 means no promtion
};

class Configuration  
{
public:
//	Configuration(MoveGen* pMoveGen);
	Configuration(Evaluator* pEvaluator, MoveGen* pMoveGen);
	Configuration(Evaluator* pEvaluator, Evaluator* pEvaluator2, MoveGen* pMoveGen, MoveGen* pMoveGen2);
	Configuration(Configuration* config);
	virtual ~Configuration();

private:
	Evaluator* m_pEvaluator;
	Evaluator* m_pEvaluator2;
	MoveGen* m_pMoveGen;
	MoveGen* m_pMoveGen2; //for capture move only

public:
	PIECE board[8][8]; //the distribution of chessmen
	PIECE valControl[8][8][4]; //the control condition: 
							//valControl[i][j][0] is the black control number; initially 0
							//valControl[i][j][1] is the least black piece who has the control
							//	initially 1000
							//valControl[i][j][2] is the white control number; initially 0
							//valControl[i][j][3] is the least white piece who has the control
							//	initially 1000
	//bitboards
	BITBOARD bK; //black king
	BITBOARD bQ; //black queen
	BITBOARD bB; //black bishop
	BITBOARD bN; //black knight
	BITBOARD bR; //black rook
	BITBOARD bP; //black pawn
	BITBOARD wK; //white king
	BITBOARD wQ; //white queen
	BITBOARD wB; //white bishop
	BITBOARD wN; //white knight
	BITBOARD wR; //white rook
	BITBOARD wP; //white pawn

	BITBOARD bOccupied; //blackOccupied
	BITBOARD wOccupied; //whiteOccupied
	BITBOARD allOccupied; //allOccupied
	BITBOARD allR90; //whiteOccupied
	BITBOARD allR45; //whiteOccupied
	BITBOARD allL45; //whiteOccupied

public:
	//some feature values of the config
	bool amIBlack; //true if the engine stands for black
	bool blackNext;   //true if currently it's the black's turn to move
	int step_count;
	Flags flags;

	int evalValue; //save the evaluation value of evaluate()
	int backup_value;		//	the real value of a node, initialized to MinValue	
	int parentEval; //eval of parent config, used to test quiescence state
	MOVE move; //save the move to produce this config

	//some important methods used mainly internally
	void cmpBitBoard(); //computer some bitboard needed in future
	void cmpControl(); //computer the control matrix

	//some public interfaces
	int evaluate();
	ConfigList* successors();	//all the successors
	Configuration* makeConfig(MOVE move); //make config for move
	
	void setEvaluator(Evaluator* evaluaor);
	bool isEndGame();

	int evaluate2();
	ConfigList* successors2(); //the capture move successors
	Configuration* makeConfig2(MOVE move); //make config for second search
	
	Configuration& operator=(Configuration& config);
	bool equal(Configuration& config);
};


#endif // !defined(AFX_CONFIGURATION_H__A76BF06E_1366_4226_B9B2_CAF884669CD9__INCLUDED_)
