// Controller.cpp: implementation of the Controller class.
//
//////////////////////////////////////////////////////////////////////

#include "Controller.h"
#include "Parser.h"
#include "Reporter.h"
#include "Analyzer.h"
#include "BitBoard.h"

#ifdef USINGHASH
#include "Hash.h"
#endif

#ifdef OPENBOOK
#include "Opengame.h"
#endif
//#include <ctime>

//for debug only
#include <iostream>
#include <fstream>
using namespace std;
//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

Controller::Controller()
{
	m_pCurConfig=NULL;
	m_pEvalOpen=NULL;
	m_pEvalMid=NULL;
	m_pEvalEnd=NULL;
	m_pEvaluator2=NULL;
	m_pMoveGen=NULL;
	m_pMoveGen2=NULL;
	
	plyLimit=4;
	nodeLimit=0;
	isTraceOff=false;
	evalType=MIDGAME;
	ColorAssigned=-1;
}

Controller::~Controller()
{
	if(m_pEvalOpen!=NULL) 
	{
		delete m_pEvalOpen;
	}
	
	if(m_pEvalMid!=NULL) 
	{
		delete m_pEvalMid;
	}
	
	if(m_pEvalEnd!=NULL) 
	{
		delete m_pEvalEnd;
	}
	
	if(m_pEvaluator2!=NULL) 
	{
		delete m_pEvaluator2;
	}

	if(m_pMoveGen!=NULL) 
	{
		delete m_pMoveGen;
	}

	if(m_pMoveGen2!=NULL) 
	{
		delete m_pMoveGen2;
	}

	if(m_pCurConfig!=NULL) 
	{
		delete m_pCurConfig;
	}

	#ifdef USINGHASH
		cleanupHash();
	#endif

}

void Controller::initialize()
{
	m_pEvalOpen=new Opening();
	m_pEvalMid=new Midgame();
	m_pEvalEnd=new EndGame();

	m_pEvaluator2=new Quiescence();

	m_pMoveGen=new MoveGen2();

	m_pMoveGen2=new MoveGen4();

	initBitBoard();

#ifdef USINGHASH
	initHash();
#endif

#ifdef OPENBOOK
	initOpenbook();
#endif

	m_pCurConfig=new Configuration(m_pEvalOpen,m_pEvaluator2,m_pMoveGen,m_pMoveGen2);

}

void Controller::start()
{
	Parser parser;
	Reporter reporter;
	Analyzer analyzer(plyLimit,nodeLimit);

	Message msg;

	//debug
	ofstream debug("test.txt", ios::app);
/*	if(isTraceOff)
	{
		msg.isTraceOff=true;
	}
	else
	{
		msg.isTraceOff=false;
	}
*/
	bool toQuit=false;
//	time_t tStart;
//	time_t tEnd;
	//loop forever until get the message to quit
	do
	{
		//Configuration* bakCurConfig=NULL;
//		int nodeSearched;
//		double timeExe;
		parser.getMessage(&msg); //blocked if no new message
		
		
				switch(msg.type)
		{
			// Send back to Acknowledge
			case INITIALIZE:
				{
					reporter.sendMessage(&msg);
				}
				break;  // Done
			// Modify Feature
			case PROTOVER:
				{
					reporter.sendMessage(&msg);
				}
				break;  // Done

			// Game is going to start, Initialize Config
			case NEW:
				{
					
				}		
				break;

			case FORCE:
				{
					// Begin to Start the Game
					if(msg.GameMode)
					{
						if(ColorAssigned==-1)
						{
							ColorAssigned=1;

							// Start Game to move 
							// So generate your first move here.
							msg.type=MOVING;

							m_pCurConfig->amIBlack=false;
							m_pCurConfig->blackNext=false;
							msg.move=analyzer.plyGen(m_pCurConfig);
							reporter.sendMessage(&msg);

						}
						else if(ColorAssigned==0)
						{
/*
	m_pCurConfig->amIBlack=true;
	m_pCurConfig->blackNext=false;
							
	m_pCurConfig->step_count=37;
	if(m_pCurConfig->step_count<26)
	{
		if ((m_pCurConfig->amIBlack) && (m_pCurConfig->flags.blackCastled))
			m_pCurConfig->setEvaluator(m_pEvalMid);
		else if ((!m_pCurConfig->amIBlack) && (m_pCurConfig->flags.whiteCastled))
			m_pCurConfig->setEvaluator(m_pEvalMid);
			 else m_pCurConfig->setEvaluator(m_pEvalOpen);
	}
	else if(m_pCurConfig->isEndGame())
	{
		m_pCurConfig->setEvaluator(m_pEvalEnd);
	}
		else m_pCurConfig->setEvaluator(m_pEvalMid);
	
	m_pCurConfig=m_pCurConfig->makeConfig(msg.move);
	msg.move=analyzer.plyGen(m_pCurConfig);
	msg.type=MOVING;
	reporter.sendMessage(&msg);
*/
							// Make your first move here.
							msg.type=MOVING;
							
							m_pCurConfig->amIBlack=true;
							m_pCurConfig->blackNext=false;
							Configuration* tmp=m_pCurConfig;
							m_pCurConfig=m_pCurConfig->makeConfig(msg.move);
							msg.move=analyzer.plyGen(m_pCurConfig);
							reporter.sendMessage(&msg);
							delete tmp;					

						}
					}
				}
				break;  // Almost done 
			
			case COLOR:
				{
					if(msg.Color)
					{
						ColorAssigned=1;
					}
					else 
					{
						ColorAssigned=0;
					}
				}
				break;	// Done

			case SETTING:
				{
					// To check the Setting Change
					if(msg.Moves!=0)
					{
						analyzer.setTimeMode(TotalTime,msg.TotalTime,msg.increTime,msg.Moves);						
					}
					else if(msg.increTime!=0)
					{
						analyzer.setTimeMode(IncreTime,msg.TotalTime,msg.increTime,msg.Moves);
					}
					else if(msg.timeperMove!=0)
					{
						analyzer.setTimeMode(PlyTime,msg.TotalTime,msg.timeperMove,msg.Moves);
					}
				}
				break;  

			case QUIT:
				{
					toQuit=true;
					
				}
				break;	

			case DRAW:
				{
					// Think about whether to draw
					// reporter.sendMessage(&msg);
					msg.type=MOVING;
					msg.move=analyzer.plyGen(m_pCurConfig);
				}
				break;

			case POST:
				{
					// Check msg.Post
					
				}
				break;	// For this version, not necessary, Done

			case HARD:
				{
					// Check msg.Difficulty
					
				}
				break;	// If hard, use evaluation, other wise simply Move Generator.

			case RESULT:
				{
					
				}
				break;	

			case MOVING:
			case PROMOTION:
				{
					// The main part here.
					if(ColorAssigned==-1)
					{
						// So we are black
						ColorAssigned=0;
					}
					else
					{
						//set evaluator
						if(m_pCurConfig->step_count<=openMaxStep)
						{
							m_pCurConfig->setEvaluator(m_pEvalOpen);
						}
						else if(m_pCurConfig->isEndGame())
						{
							m_pCurConfig->setEvaluator(m_pEvalEnd);
						}
							else m_pCurConfig->setEvaluator(m_pEvalMid);
						
					// Thinking here.
						Configuration* tmp=m_pCurConfig;

						//added for promotion
						if(PROMOTION==msg.type)
						{
							m_pCurConfig->flags.promotion=msg.promotionChoice;
						}
						m_pCurConfig=m_pCurConfig->makeConfig(msg.move);
						msg.move=analyzer.plyGen(m_pCurConfig);
						if(m_pCurConfig->flags.promotion!=0) //if promotion happens
						{
							msg.type=PROMOTION;
							msg.promotionChoice=m_pCurConfig->flags.promotion;
						}
						else
						{
							msg.type=MOVING;
						}
						if(msg.move==(MOVE)0xffff) //resign
						{
							msg.type=RESIGN;
							debug<<"Resign"<<endl;
						}
						reporter.sendMessage(&msg);
						delete tmp;
		/*				//debug
						for(int i=0;i<8;i++)
						{
							for(int j=0;j<8;j++)
							{
								debug<<m_pCurConfig->board[i][j]<<",";
							}
							debug<<endl;
						}
	*/
					}
				}
				break;
			
			case USELESS:
				{
					
				}
				break;  // Done
			
			default:
				break;	// Done
		}



	}while(!toQuit);
}
