// Evaluator.cpp: implementation of the Evaluator class.
//
//////////////////////////////////////////////////////////////////////
#include "chessRef.h"

#ifdef DEBUG
#include <fstream>
#include <string>
using namespace std;
#endif

#include "Evaluator.h"

bool Evaluator::IsWinning(Configuration* conf, bool Blackturn)
//	check if I can eat the king of the opponent in this turn
{
	if (Blackturn)
		if (conf->valControl[(pickFirst(conf->wK))/8][(pickFirst(conf->wK))%8][0]>0)
			return true;		//	check if can eat white's king
		else return false;
	else if (conf->valControl[(pickFirst(conf->bK))/8][(pickFirst(conf->bK))%8][2]>0)
			return true;		//	check if can eat black's king
		 else return false;
}

float Evaluator::King_safety(Configuration* conf, bool Blackturn)
{
	float WK_value=0;
	float BK_value=0;
	int WK_position=-1;
	int BK_position=-1;
	int WK_x=-1;
	int WK_y=-1;
	int BK_x=-1;
	int BK_y=-1;

	BK_position=pickFirst(conf->bK);		//	get the black king's position
	BK_x=BK_position/8;			//	black King's x value
	BK_y=BK_position%8;			//	black King's y value
	
	WK_position=pickFirst(conf->wK);		//	get the black king's position
	WK_x=WK_position/8;			//	white King's x value
	WK_y=WK_position%8;			//	white King's y value
	
	//	1. Being checked

	if (conf->valControl[BK_x][BK_y][2]>0)
		BK_value=BK_value-0.5;			//	if black King is being checked
	
	if (conf->valControl[WK_x][WK_y][0]>0)
		WK_value=WK_value-0.5;			//	if  white King is being checked


	//	2. pieces near the King
	
	//	Compute the short distance castle values for black
	
	if (BK_x!=0) BK_value=BK_value-0.4;					//	keep the King in the bottom line
		
	if (((conf->flags.blackCastled) || (conf->step_count>26)) &&
		(BK_x <=3) && (BK_y >=4))						//	King not in the opponent's board
	{
		if (conf->board[BK_x +2][BK_y]>128)				//	if extra things before the King
			BK_value=BK_value+0.3;
		if (conf->board[BK_x +3][BK_y]>128)				//	black pieces
			BK_value=BK_value+0.1;

		if (conf->board[BK_x +1][BK_y]!=bPawn)			//	the pawn right upon the King lost
		{
			BK_value=BK_value-0.8;		
			if (conf->board[BK_x +1][BK_y]!=0)			//	something before the King
			{
				if (conf->board[BK_x +1][BK_y]==bQueen)
					BK_value=BK_value+0.4;
				if (conf->board[BK_x +1][BK_y]==bRook)
					BK_value=BK_value+0.6;
				if (conf->board[BK_x +1][BK_y]==bBishop)
					BK_value=BK_value+0.5;
				if (conf->board[BK_x +1][BK_y]==bKnight)
					BK_value=BK_value+0.5;
			}
		}
		
		if (BK_y<7)
		{
			if (conf->board[BK_x +2][BK_y +1]>128)		//	something before the King
				BK_value=BK_value+0.2;
			if (conf->board[BK_x +3][BK_y +1]>128)
				BK_value=BK_value+0.1;
			if (conf->board[BK_x +1][BK_y +1]!=bPawn)
			{
				BK_value=BK_value-0.8;	//	the pawn upon the king and close to the boarder lost
				if ((conf->board[BK_x +2][BK_y +1]==bPawn) &&
					(conf->board[BK_x +1][BK_y]==bPawn))
					BK_value=BK_value+0.9;		//	like g7-h6 pawn links
				else if (conf->board[BK_x +1][BK_y +1]>128)
					BK_value=BK_value+0.5;						//	something before the King
			}
		}

		if (conf->board[BK_x +2][BK_y -1]>128)			//	something before the King
			BK_value=BK_value+0.2;
		if (conf->board[BK_x +3][BK_y -1]>128)
			BK_value=BK_value+0.1;
		if ((BK_y>4) && (conf->board[BK_x +1][BK_y -1]!=bPawn))
		{
			BK_value=BK_value-0.5;		//	the pawn upon the king and close to the center lost
			// if (conf->board[BK_x +1][BK_y -1]>128)
			//	BK_value=BK_value+0.5;	//	something before the King
		}
	}	//	end short castle for black

	//	Compute the long distance castle values for black

	if (((conf->flags.blackCastled) || (conf->step_count>26)) && 
		(BK_x <=3) && (BK_y <=3))						//	King not in the opponent's board
	{
		if (conf->board[BK_x +2][BK_y]>128)				//	if extra things before the King
			BK_value=BK_value+0.3;
		if (conf->board[BK_x +3][BK_y]>128)				//	black pieces
			BK_value=BK_value+0.1;

		if (conf->board[BK_x +1][BK_y]!=bPawn)			//	the pawn right upon the King lost
		{
			BK_value=BK_value-0.8;		
			if (conf->board[BK_x +1][BK_y]!=0)			//	something before the King
			{
				if (conf->board[BK_x +1][BK_y]==bQueen)
					BK_value=BK_value+0.4;
				if (conf->board[BK_x +1][BK_y]==bRook)
					BK_value=BK_value+0.6;
				if (conf->board[BK_x +1][BK_y]==bBishop)
					BK_value=BK_value+0.5;
				if (conf->board[BK_x +1][BK_y]==bKnight)
					BK_value=BK_value+0.5;
			}
		}
		
		if (BK_y>0) 
		{
			if (conf->board[BK_x +2][BK_y -1]>128)		//	something before the King
				BK_value=BK_value+0.2;
			if (conf->board[BK_x +3][BK_y -1]>128)
				BK_value=BK_value+0.1;
			if (conf->board[BK_x +1][BK_y -1]!=bPawn)
			{
				BK_value=BK_value-0.8;	//	the pawn upon the king and close to the boarder lost
				if (conf->board[BK_x +1][BK_y -1]>128)
					BK_value=BK_value+0.5;				//	something before the King
			}
		}

		if (conf->board[BK_x +2][BK_y +1]>128)			//	something before the King
			BK_value=BK_value+0.2;
		if (conf->board[BK_x +3][BK_y +1]>128)
			BK_value=BK_value+0.1;
		if ((BK_y>4) && (conf->board[BK_x +1][BK_y +1]!=bPawn))
		{
			BK_value=BK_value-0.2;		//	the pawn upon the king and close to the center lost
			//if (conf->board[BK_x +1][BK_y +1]>128)
			//	BK_value=BK_value+0.2;	//	something before the King
		}
	}	//	end long castle for black

	//	Compute the short distance castle values for white

	if (WK_x!=7) WK_value=WK_value-0.4;							//	keep the King in the bottom line
		
	if (((conf->flags.whiteCastled) || (conf->step_count>26)) && 
		(WK_x >=4)	&& (WK_y>=4))						//	King not in the opponent's board
	{
		if ((conf->board[WK_x -2][WK_y]<128) &&
			(conf->board[WK_x -2][WK_y]!=0))			//	if extra things before the King
			WK_value=WK_value+0.3;
		if ((conf->board[WK_x -3][WK_y]<128) &&
			(conf->board[WK_x -3][WK_y]!=0))			//	white pieces
			WK_value=WK_value+0.1;

		if (conf->board[WK_x -1][WK_y]!=wPawn)			//	the pawn right upon the King lost
		{
			WK_value=WK_value-0.8;		
			if (conf->board[WK_x -1][WK_y]!=0)			//	something before the King
			{
				if (conf->board[WK_x -1][WK_y]==wQueen)
					WK_value=WK_value+0.4;
				if (conf->board[WK_x -1][WK_y]==wRook)
					WK_value=WK_value+0.6;
				if (conf->board[WK_x -1][WK_y]==wBishop)
					WK_value=WK_value+0.5;
				if (conf->board[WK_x -1][WK_y]==wKnight)
					WK_value=WK_value+0.5;
			}
		}
		
		if (WK_y<7)
		{
			if ((conf->board[WK_x -2][WK_y +1]<128) &&
				(conf->board[WK_x -2][WK_y +1]!=0))		//	something before the King
				WK_value=WK_value+0.2;
			if ((conf->board[WK_x -3][WK_y +1]<128) &&
				(conf->board[WK_x -3][WK_y +1]!=0))
				WK_value=WK_value+0.1;
			if (conf->board[WK_x -1][WK_y +1]!=wPawn)
			{
				WK_value=WK_value-0.8;	//	the pawn upon the king and close to the boarder lost
				if ((conf->board[WK_x -2][WK_y +1]==wPawn) &&
					(conf->board[WK_x -1][WK_y]==wPawn))	//	white pawn links like g2-h3
					WK_value=WK_value+0.9;
				else if ((conf->board[WK_x -1][WK_y +1]<128) &&
						 (conf->board[WK_x -1][WK_y +1]!=0))
						WK_value=WK_value+0.5;			//	something before the King
			}
		}

		if ((conf->board[WK_x -2][WK_y -1]<128) &&
			(conf->board[WK_x -2][WK_y -1]!=0))			//	something before the King
			WK_value=WK_value+0.2;
		if ((conf->board[WK_x -3][WK_y -1]<128) &&
			(conf->board[WK_x -3][WK_y -1]!=0))
			WK_value=WK_value+0.1;
		if ((WK_y>4) && (conf->board[WK_x -1][WK_y -1]!=wPawn))
		{
			WK_value=WK_value-0.5;		//	the pawn upon the king and close to the center lost
			//if ((conf->board[WK_x -1][WK_y -1]<128) &&
			//	(conf->board[WK_x -1][WK_y -1]!=0))
			//	WK_value=WK_value+0.5;	//	something before the King
		}
	}	//	end short castle for white

	//	Compute the long distance castle values for white

	if (((conf->flags.whiteCastled) || (conf->step_count>26)) && 
		(WK_x >=4)	&& (WK_y<=3))						//	King not in the opponent's board
	{
		if ((conf->board[WK_x -2][WK_y]<128) &&
			(conf->board[WK_x -2][WK_y]!=0))			//	if extra things before the King
			WK_value=WK_value+0.3;
		if ((conf->board[WK_x -3][WK_y]<128) &&
			(conf->board[WK_x -3][WK_y]!=0))			//	black pieces
			WK_value=WK_value+0.1;

		if (conf->board[WK_x -1][WK_y]!=wPawn)			//	the pawn right upon the King lost
		{
			WK_value=WK_value-0.8;		
			if (conf->board[WK_x -1][WK_y]!=0)			//	something before the King
			{
				if (conf->board[WK_x -1][WK_y]==wQueen)
					WK_value=WK_value+0.4;
				if (conf->board[WK_x -1][WK_y]==wRook)
					WK_value=WK_value+0.6;
				if (conf->board[WK_x -1][WK_y]==wBishop)
					WK_value=WK_value+0.5;
				if (conf->board[WK_x -1][WK_y]==wKnight)
					WK_value=WK_value+0.5;
			}
		}
		
		if (WK_y>0)
		{
			if ((conf->board[WK_x -2][WK_y -1]<128) &&
				(conf->board[WK_x -2][WK_y -1]!=0))			//	something before the King
				WK_value=WK_value+0.2;
			if ((conf->board[WK_x -3][WK_y -1]<128) &&
				(conf->board[WK_x -3][WK_y -1]!=0))
				WK_value=WK_value+0.1;
			if (conf->board[WK_x -1][WK_y -1]!=wPawn)
			{
				WK_value=WK_value-0.8;	//	the pawn upon the king and close to the boarder lost
				if ((conf->board[WK_x -1][WK_y -1]<128) &&
					(conf->board[WK_x -1][WK_y -1]!=0))
					WK_value=WK_value+0.5;					//	something before the King
			}
		}

		if ((conf->board[WK_x -2][WK_y +1]<128) &&
			(conf->board[WK_x -2][WK_y +1]!=0))				//	something before the King
			WK_value=WK_value+0.2;
		if ((conf->board[WK_x -3][WK_y +1]<128) &&
			(conf->board[WK_x -3][WK_y +1]!=0))
			WK_value=WK_value+0.1;
		if ((WK_y>4) && (conf->board[WK_x -1][WK_y +1]!=wPawn))
		{
			WK_value=WK_value-0.2;		//	the pawn upon the king and close to the center lost
			//if ((conf->board[WK_x -1][WK_y +1]<128) &&
			//	(conf->board[WK_x -1][WK_y +1]!=0))
			//	WK_value=WK_value+0.2;	//	something before the King
		}
	}	//	end long castle for white

	if (Blackturn) return(BK_value-WK_value);
	else return(WK_value-BK_value);
}

float Evaluator::Materials(Configuration* conf, bool Blackturn)
{
	int BM_value=0;
	int WM_value=0;
	
	for (int i=0;i<8;i++)
		for (int j=0;j<8;j++)
			switch (conf->board[i][j])
			{
			case wKing:		WM_value+=250;	
							PieceBoard[i][j]=250;
							break;
			case wQueen:	WM_value+=9;
							PieceBoard[i][j]=9;
							break;
			case wRook:		WM_value+=5;	
							PieceBoard[i][j]=5;
							break;
			case wBishop:	WM_value+=3;	
							PieceBoard[i][j]=3;
							break;
			case wKnight:	WM_value+=3;	
							PieceBoard[i][j]=3;
							break;
			case wPawn:		WM_value+=1;	
							PieceBoard[i][j]=1;
							break;
			case bKing:		BM_value+=250;	
							PieceBoard[i][j]=400+250;
							break;
			case bQueen:	BM_value+=9;	
							PieceBoard[i][j]=400+9;
							break;
			case bRook:		BM_value+=5;	
							PieceBoard[i][j]=400+5;
							break;
			case bBishop:	BM_value+=3;
							PieceBoard[i][j]=400+3;
							break;
			case bKnight:	BM_value+=3;
							PieceBoard[i][j]=400+3;
							break;
			case bPawn:		BM_value+=1;
							PieceBoard[i][j]=400+1;
							break;
			default:		PieceBoard[i][j]=0;
							break;
			}

	if (Blackturn) return(BM_value-WM_value);
	else return(WM_value-BM_value);
}

float Evaluator::Attacking(Configuration* conf, bool Blackturn)
{
	float BA_value=0;
	float WA_value=0;
	float temp=0;
	
	for (int i=0;i<8;i++)
		for (int j=0;j<8;j++)
		{
			if (Blackturn)
			{
				if (conf->valControl[i][j][0]>conf->valControl[i][j][2])
					//	black more controlled
					if ((PieceBoard[i][j]==0) || (PieceBoard[i][j]>=400))	
					//	a blank square or a black piece in the square
					{
						if (PieceBoard[i][j]==0) 
						//	a blank square controled by more black
							BA_value=BA_value+0.07;	
						else if (PieceBoard[i][j]-400<=conf->valControl[i][j][3])	
								 BA_value=BA_value+0.07;			//	a safe black piece
							 else WA_value=WA_value+0.10;			//	an unsafe black piece
					}
					//	a white piece in this square
					else if (conf->valControl[i][j][2]==0)	
							 BA_value=BA_value+PieceBoard[i][j];	//	no white piece protects this piece
						 else if ((temp=conf->valControl[i][j][1]-conf->valControl[i][j][3])<0) 
								  // just eat the white piece in this square, black worth
								  BA_value=BA_value+PieceBoard[i][j];	
							  else {
								      if ((temp=PieceBoard[i][j]-temp)>0) BA_value=BA_value+temp;
								  	  // if 1 black exchange with 2 white is worth
									  else WA_value=WA_value+0.07;	//	an safe white piece
									}

				//	white more or equal controlled
				else if (PieceBoard[i][j]<400)
				//	a blank square or a white piece in the square
				{
					if (PieceBoard[i][j]==0) 
					{	//	a blank square
						if (conf->valControl[i][j][0]<conf->valControl[i][j][2])
							WA_value=WA_value+0.07;		//	a blank square controled by more white
						else;		//	a blank square with equal controlling
					}
					//	a white piece in the square
					else if ((temp=PieceBoard[i][j]-conf->valControl[i][j][1])<=0)
							 WA_value=WA_value+0.07;	    //	a safe white piece in the square
						 else BA_value=BA_value+temp;	// an unsafe white piece, black will eat it
				}
					//	a black piece in this square
					else if (conf->valControl[i][j][0]==0)//	no black piece protects this piece
					{ 
						 if (conf->valControl[i][j][2]>0) 
							 WA_value=WA_value+0.10;		  //	this black piece is being attacked
						 else BA_value=BA_value+0.07;	  //	a safe black piece
					}		
						 else if (conf->valControl[i][j][0]<conf->valControl[i][j][2])
						 {	  
							  //	controled by more white
							  if (PieceBoard[i][j]-400+	
								  conf->valControl[i][j][1]-conf->valControl[i][j][3]>0) 
							      WA_value=WA_value+0.10;	 //	white's piece value <black 2 pieces'
							  else BA_value=BA_value+0.07;	 //	a safe black piece
						 }
							  //	equal protection
							  else if (PieceBoard[i][j]-400-conf->valControl[i][j][3]>0)
									   WA_value=WA_value+0.10;	// 1 white exchanges 1 black, worth
								   else BA_value=BA_value+0.07;	//	a safe black piece
			}	//	end if Blackturn
			

			///////////////////////////////////////////////////////////////////////////
			//	white's turn
			///////////////////////////////////////////////////////////////////////////

			else
			{
				if (conf->valControl[i][j][0]<conf->valControl[i][j][2])
				{
					//	white more controlled
					if (PieceBoard[i][j]<400)
					//	a blank square or a white piece in this square
					{
						if (PieceBoard[i][j]==0) 
							WA_value=WA_value+0.07;			//	a blank square
						else if (PieceBoard[i][j]<=conf->valControl[i][j][3])	
								 WA_value=WA_value+0.07;	    //	a safe white piece
							 else BA_value=BA_value+0.10;	//	a unsafe white piece
					}
					//	a black piece in this square
					else if (conf->valControl[i][j][0]==0)	
							 WA_value=WA_value+PieceBoard[i][j]-400;	//	no black piece protects this piece
						 else if ((temp=conf->valControl[i][j][3]-conf->valControl[i][j][1])<0) 
								  // just eat the black piece in this square
								  WA_value=WA_value+PieceBoard[i][j]-400;
							  else {
								      if ((temp=PieceBoard[i][j]-400-temp)>0) 
										  WA_value=WA_value+temp; // 1 white exchange with 2 black is worth
									  else BA_value=BA_value+0.07; // a safe black piece
									}
				}
				//	black more or equal
				else if ((PieceBoard[i][j]==0) || (PieceBoard[i][j]>=400))
				//	a blank square or a black piece in the square
				{
					if (PieceBoard[i][j]==0) 
					//	a blank square
					{	
						if (conf->valControl[i][j][2]<conf->valControl[i][j][0])
							BA_value=BA_value+0.07;		//	a blank square controled by more black
						else;		//	equally controlled
					}
					//	a black piece in the square
					else if ((temp=PieceBoard[i][j]-400-conf->valControl[i][j][3])<=0)
							 BA_value=BA_value+0.07;			 //	a safe black piece in the square
						 else WA_value=WA_value+temp;		 //	an unsafe black piece eaten by white
													 
				}
					 //	a white piece in this square
					 else if (conf->valControl[i][j][2]==0)	 //	no white piece protects this piece
					 {
						 if (conf->valControl[i][j][0]>0)
							 BA_value=BA_value+0.10;			 //	this white piece is being attacked
						 else WA_value=WA_value+0.07;		 //	a safe white piece
					 }	
						 else if (conf->valControl[i][j][2]<conf->valControl[i][j][0])
						 {	  
							  //	controled by more black
							  if (PieceBoard[i][j]+
								  conf->valControl[i][j][3]-conf->valControl[i][j][1]>0) 
							      BA_value=BA_value+0.10;	 //	black's piece value < 2 white pieces'
							  else WA_value=WA_value+0.07;	 //	a safe white piece
						  }
						      //	equal protection
						      else if (PieceBoard[i][j]
										-conf->valControl[i][j][1]>0)	//	1 black exchanges 1 white
										BA_value=BA_value+0.10;			//	worth
									else WA_value=WA_value+0.07;			//	a safe white piece
			}	//	end if (Blackturn) else	
		}	//	end for
	
	if (Blackturn) return(BA_value-WA_value);
	else return(WA_value-BA_value);
}

float Evaluator::Board_control(Configuration* conf, bool Blackturn)
{
	float BB_value=0;
	float WB_value=0;

	if ((conf->valControl[3][3][2]-conf->valControl[3][3][0])>0)
		WB_value+=(float)0.1;
	else if ((conf->valControl[3][3][2]-conf->valControl[3][3][0])<0)
		BB_value+=(float)0.1;

	if ((conf->valControl[3][4][2]-conf->valControl[3][4][0])>0)
		WB_value+=(float)0.1;
	else if ((conf->valControl[3][4][2]-conf->valControl[3][4][0])<0)
		BB_value+=(float)0.1;

	if ((conf->valControl[4][3][2]-conf->valControl[4][3][0])>0)
		WB_value+=(float)0.1;
	else if((conf->valControl[4][3][2]-conf->valControl[4][3][0])<0)
		BB_value+=(float)0.1;

	if ((conf->valControl[4][4][2]-conf->valControl[4][4][0])>0)
		WB_value+=(float)0.1;
	else if ((conf->valControl[4][4][2]-conf->valControl[4][4][0])<0)
		BB_value+=(float)0.1;

	 for (int i=3;i<5;i++)
		for (int j=3;j<5;j++)
		{
			if (conf->board[i][j]>128)
				BB_value+=(float)0.1;	//	a black piece
			else if (conf->board[i][j]!=0) 
				WB_value+=(float)0.1;	//	a white piece
		}

	if (Blackturn) return(BB_value-WB_value);
	else return(WB_value-BB_value);
}

float Evaluator::King_center(Configuration* conf, bool Blackturn)
{
	float WK_value=0;
	float BK_value=0;
	int WK_position=-1;
	int BK_position=-1;
	int WK_x=-1;
	int WK_y=-1;
	int BK_x=-1;
	int BK_y=-1;

	BK_position=pickFirst(conf->bK);		//	get the black king's position
	BK_x=BK_position/8;						//	black King's x value
	BK_y=BK_position%8;						//	black King's y value
	
	WK_position=pickFirst(conf->wK);		//	get the black king's position
	WK_x=WK_position/8;						//	white King's x value
	WK_y=WK_position%8;						//	white King's y value
	
	//	evaluate the King's position

	if (BK_x<=2)
		BK_value=BK_value-0.1*(3-BK_x);		//	black King's position is low
	if (WK_x>=5)
		WK_value=WK_value-0.1*(WK_x-4);		//	white King's position is low

	//	if there are any white pawns of the opponent around the black King

	switch(BK_y)
	{
	case 0:		if (conf->board[BK_x][BK_y+1]==wPawn)
					BK_value=BK_value+0.2;		//	a white pawn beside the King
				if (BK_x != 7)
					if (conf->board[BK_x+1][BK_y]==wPawn)
						BK_value=BK_value+0.2;	//	a white pawn in front of the King
				break;

	case 7:		if (conf->board[BK_x][BK_y-1]==wPawn)
					BK_value=BK_value+0.2;		//	a white pawn beside the King
				if (BK_x != 7)
					if (conf->board[BK_x+1][BK_y]==wPawn)
						BK_value=BK_value+0.2;	//	a white pawn in front of the King
				break;
	
	default:	if (conf->board[BK_x][BK_y+1]==wPawn)
					BK_value=BK_value+0.2;		//	a white pawn beside the King
				if (conf->board[BK_x][BK_y-1]==wPawn)
					BK_value=BK_value+0.2;		//	a white pawn beside the King
				if (BK_x != 7)
					if (conf->board[BK_x+1][BK_y]==wPawn)
						BK_value=BK_value+0.2;	//	a white pawn in front of the King
				break;
	}

	//	if there are any black pawns of the opponent around the white King

	switch(WK_y)
	{
	case 0:		if (conf->board[WK_x][WK_y+1]==bPawn)
					WK_value=WK_value+0.2;		//	a black pawn beside the King
				if (WK_x != 0)
					if (conf->board[WK_x-1][WK_y]==bPawn)
						WK_value=WK_value+0.2;	//	a black pawn in front of the King
				break;

	case 7:		if (conf->board[WK_x][WK_y-1]==bPawn)
					WK_value=WK_value+0.2;		//	a black pawn beside the King
				if (WK_x != 0)
					if (conf->board[WK_x-1][WK_y]==bPawn)
						WK_value=WK_value+0.2;	//	a black pawn in front of the King
				break;
	
	default:	if (conf->board[WK_x][WK_y+1]==bPawn)
					WK_value=WK_value+0.2;		//	a black pawn beside the King
				if (conf->board[WK_x][WK_y-1]==bPawn)
					WK_value=WK_value+0.2;		//	a black pawn beside the King
				if (WK_x != 0)
					if (conf->board[WK_x-1][WK_y]==bPawn)
						WK_value=WK_value+0.2;	//	a black pawn in front of the King
				break;
	}
	
	if (Blackturn) return(BK_value-WK_value);
	else return(WK_value-BK_value);
}


float Evaluator::Pawn_position(Configuration* conf, bool Blackturn)
{
	float BP_value=0;
	float WP_value=0;
	int BP_position=-1;
	int WP_position=-1;
	int BP_x=-1;
	int BP_y=-1;
	int WP_x=-1;
	int WP_y=-1;
	float temp=0;
	BITBOARD P;

	P=conf->bP;									//	get the pawn's BitBoard
	while(P!=0)
	{
		BP_position=pickFirst(P);				//	get the black pawn's position
		BP_x=BP_position/8;						//	black pawn's x value
		BP_y=BP_position%8;						//	black pawn's y value

		switch(BP_x)
		{
		case 6:		if (Blackturn)
					{
						if (conf->board[7][BP_y]==0)	
							//	the bottom square is blank
							if (conf->valControl[7][BP_y][0]>=conf->valControl[7][BP_y][2])
							{
								//	the bottom square controled by more black 
								if (conf->valControl[7][BP_y][2]==0)
									BP_value=BP_value+QueenValue;	//	queen promotion
								else BP_value=BP_value+conf->valControl[7][BP_y][3]-1.5;
								//	this black pawn exchanges with a white piece
							}
							//	the bottom square controled by more white
							else if ((temp=conf->valControl[7][BP_y][1]+(float)1.5-
									 conf->valControl[7][BP_y][3])<0)
								//	this black pawn plus 1 black piece exchanges with a white piece
									{
										BP_value=BP_value+temp;
									}
								 else 	BP_value=BP_value+1;	//	black can't be promoted
						else BP_value=BP_value+0.5;				//	a white piece ahead
					}	//	end if Blackturn
					
					else	//	white turn
					{
						if (conf->valControl[6][BP_y][0]>=conf->valControl[6][BP_y][2])
							{
								//	the black pawn is controled by more black pieces
								BP_value=BP_value+1;			//	a safe black pawn
							}
						else if ((temp=conf->valControl[6][BP_y][1]+(float)1.5-
								 conf->valControl[6][BP_y][3])<0)
								{
									BP_value=BP_value+1;		//	a safe black pawn
								}
					}	//	end else-if Blackturn
					break;
	
		case 5:		if (conf->valControl[5][BP_y][0]>=conf->valControl[5][BP_y][2])
					{
						BP_value=BP_value+0.2;			//	a safe black pawn
					}
					break;

		case 4:		if (conf->valControl[5][BP_y][0]>=conf->valControl[5][BP_y][2])
					{
						BP_value=BP_value+0.1;			//	a safe black pawn
					}
		default:	break;
		}	//	end switch

		P&=~(Unit<<BP_position);
	}	//	end while
			

	P=conf->wP;									//	get the white pawn's BitBoard
	while(P!=0)
	{
		WP_position=pickFirst(P);				//	get the white pawn's position
		WP_x=WP_position/8;						//	white pawn's x value
		WP_y=WP_position%8;						//	white pawn's y value

		switch(WP_x)
		{
		case 1:		if (!Blackturn)
					{
						if (conf->board[0][WP_y]==0)	
							//	the bottom square is blank
							if (conf->valControl[0][WP_y][2]>=conf->valControl[0][WP_y][0])
							{
								//	the bottom square controled by more white 
								if (conf->valControl[0][WP_y][0]==0)
									WP_value=WP_value+QueenValue;	//	queen promotion
								else WP_value=WP_value+conf->valControl[0][WP_y][1]-1.5;
								//	this white pawn exchanges with a black piece
							}
							//	the bottom square controled by more black
							else 
							{
								if ((temp=conf->valControl[0][WP_y][3]+(float)1.5-
									 conf->valControl[0][WP_y][1])<0)
								//	this white pawn plus 1 white piece exchanges with a black piece
									{
										WP_value=WP_value+temp;
									}
								 else 	WP_value=WP_value+1;	//	white can't be promoted
							}
						else WP_value=WP_value+0.5;				//	a piece ahead
					}	//	end if Blackturn
					
					else	//	black turn
					{
						if (conf->valControl[1][WP_y][2]>=conf->valControl[1][WP_y][0])
							{
								//	the white pawn is controled by more white pieces
								WP_value=WP_value+1;			//	a safe white pawn
							}
						else if ((temp=conf->valControl[1][WP_y][3]+(float)1.5-
								 conf->valControl[1][WP_y][1])<0)
								{
									WP_value=WP_value+1;		//	a safe white pawn
								}
					}	//	end else-if !Blackturn
					break;
	
		case 2:		if (conf->valControl[2][WP_y][2]>=conf->valControl[2][WP_y][0])
					{
						WP_value=WP_value+0.2;			//	a safe white pawn
					}
					break;

		case 3:		if (conf->valControl[3][WP_y][2]>=conf->valControl[3][WP_y][0])
					{
						WP_value=WP_value+0.1;			//	a safe white pawn
					}
		default:	break;
		}	//	end switch

		P&=~(Unit<<WP_position);
	}	//	end while
	
	if (Blackturn) return(BP_value-WP_value);
	else return(WP_value-BP_value);
}


float Evaluator::Opening_rules(Configuration* conf, bool Blackturn)
{
	float BO_value=0;
	float WO_value=0;
	
	//	castle as soon as possible

	if ((conf->flags.leftBlackRookMoved || conf->flags.blackKingMoved) && 
		(conf->flags.rightBlackRookMoved || conf->flags.blackKingMoved) && 
		!(conf->flags.blackCastled)) 
	{
	//	black has no chance to castle
		if (conf->wQ!=0)
			BO_value=BO_value-0.9;		//	white queen exists
		else BO_value=BO_value-0.4;		//	no white queen
	}
	else if ((conf->step_count>6) && !(conf->flags.blackCastled))
		 BO_value=BO_value-0.55;			//	black still not castle
	
	if ((conf->flags.leftWhiteRookMoved || conf->flags.whiteKingMoved) && 
		(conf->flags.rightWhiteRookMoved || conf->flags.whiteKingMoved) && 
		!(conf->flags.whiteCastled)) 
	{
	//	white has no chance to castle
		if (conf->bQ!=0)
			WO_value=WO_value-0.9;		//	black queen exists	
		else WO_value=WO_value-0.4;		//	no black queen
	}
	else if ((conf->step_count>6) && !(conf->flags.whiteCastled))
   		     WO_value=WO_value-0.55;		//	white still no castle

	//	Knights and Bishops should move out

	if (conf->board[7][1]!=wKnight)
		WO_value+=(float)0.3;
	if (conf->board[7][2]!=wBishop)
		WO_value+=(float)0.25;
	if (conf->board[7][6]!=wKnight)
		WO_value+=(float)0.3;
	if (conf->board[7][5]!=wBishop)
		WO_value+=(float)0.25;
		
	if (conf->board[0][1]!=bKnight)
		BO_value+=(float)0.3;
	if (conf->board[0][2]!=bBishop)
		BO_value+=(float)0.25;
	if (conf->board[0][6]!=bKnight)
		BO_value+=(float)0.3;
	if (conf->board[0][5]!=bBishop)
		BO_value+=(float)0.25;
	
	//	avoid moving knights to a or h lines

	if (conf->board[2][0]==bKnight)
		BO_value-=(float)0.25;
	if (conf->board[2][7]==bKnight)
		BO_value-=(float)0.25;
	if (conf->board[5][0]==wKnight)
		WO_value-=(float)0.25;
	if (conf->board[5][7]==wKnight)
		WO_value-=(float)0.25;	

	//	queen should not move

	if (conf->step_count<21)
	{
		if ((conf->wQ!=0) && (conf->board[7][3]==wQueen))
			WO_value=WO_value+0.7;
	
		if ((conf->bQ!=0) && (conf->board[0][3]==bQueen))
			BO_value=BO_value+0.7;
	}
	else
	{
		if ((conf->wQ!=0) && (conf->board[7][3]==wQueen))
			WO_value=WO_value+0.4;
	
		if ((conf->bQ!=0) && (conf->board[0][3]==bQueen))
			BO_value=BO_value+0.4;
	}

	if (Blackturn) return(BO_value-WO_value);
	else return(WO_value-BO_value);
}


float Evaluator::Ending_rules(Configuration* conf, bool Blackturn)
{
	float BE_value=0;
	float WE_value=0;
	int BP_position=-1;
	int WP_position=-1;
	int BP_x=-1;
	int BP_y=-1;
	int WP_x=-1;
	int WP_y=-1;
	float temp=0;
	BITBOARD P;


	P=conf->wP;									//	get the white pawn's BitBoard
	while(P!=0)
	{
		WP_position=pickFirst(P);				//	get the white pawn's position
		WP_x=WP_position/8;						//	white pawn's x value
		WP_y=WP_position%8;						//	white pawn's y value

		if (conf->valControl[WP_x][WP_y][0]>0)	//	some black piece attacking
			BE_value= BE_value+0.1;				//	this white pawn
		if (conf->valControl[WP_x -1][WP_y][0]>0)	//	some black piece controlling
			BE_value= BE_value+0.1;					//	the square ahead of white pawn
		if (conf->board[WP_x -1][WP_y]>128)		//	some black piece occupying
			BE_value= BE_value+0.1;				//	the square ahead of white pawn

		P&=~(Unit<<WP_position);
	}	//	end while

	P=conf->bP;									//	get the black pawn's BitBoard
	while(P!=0)
	{
		BP_position=pickFirst(P);				//	get the black pawn's position
		BP_x=BP_position/8;						//	black pawn's x value
		BP_y=BP_position%8;						//	black pawn's y value

		if (conf->valControl[BP_x][BP_y][2]>0)	//	some white piece attacking
			WE_value= WE_value+0.1;				//	this black pawn
		if (conf->valControl[BP_x +1][BP_y][2]>0)	//	some white piece controlling
			WE_value= WE_value+0.1;					//	the square ahead of black pawn
		if ((conf->board[BP_x -1][BP_y]<128) &&
			(conf->board[BP_x -1][BP_y]!=0))		//	some white piece occupying
			WE_value= WE_value+0.1;				//	the square ahead of black pawn

		P&=~(Unit<<BP_position);
	}	//	end while
	
	if (Blackturn) return(BE_value-WE_value);
	else return(WE_value-BE_value);
}


int Opening::evaluate(Configuration* config)
{
	float evalValue=0;

	if(IsWinning(config, config->blackNext))	
//	   if (config->amIBlack==config->blackNext) evalValue=2000;	// return the maximum value if winning
//	   else evalValue=-2000;					//	the opponent wins, return minimum value
		evalValue=MaxEvalValue;
	//	evaluate 5 factors according to the current configuration
	else
	{
		//	k= King_safety(config, config->blackNext);
		if (((config->amIBlack) && config->flags.blackCastled) ||
			((!config->amIBlack) && config->flags.whiteCastled))
			k= King_safety(config, config->blackNext);
		else k=0;
		m= Materials(config, config->blackNext);
		a= Attacking(config, config->blackNext);
		b= Board_control(config, config->blackNext);
		o= Opening_rules(config, config->blackNext);

		evalValue=100*(k+m+a+b+o);
	}
	
	if (config->amIBlack==config->blackNext) 
		if (evalValue> MaxEvalValue) 
			return MaxEvalValue;
			else return (int)(evalValue); //+0.0001 to avoid incorrect ignorance 
	else if (-evalValue< MinEvalValue)
			return MinEvalValue;
		else return (int)(-evalValue);
}


int Midgame::evaluate(Configuration* config)
{
	float evalValue=0;

	if(IsWinning(config, config->blackNext))	
//	   if (config->amIBlack==config->blackNext) evalValue=2000;	// return the maximum value if winning
//	   else evalValue=-2000;					//	the opponent wins, return minimum value
		evalValue=MaxEvalValue;
	//	evaluate 4 factors according to the current configuration
	else
	{
		k= King_safety(config, config->blackNext);
		m= Materials(config, config->blackNext);
		a= Attacking(config, config->blackNext);
		b= Board_control(config, config->blackNext);
		p= Pawn_position(config, config->blackNext);

		evalValue=100*(k+m+a+b+p);
	}
	
	if (config->amIBlack==config->blackNext) 
		if (evalValue> MaxEvalValue) 
			return MaxEvalValue;
			else return (int)(evalValue); //+0.0001 to avoid incorrect ignorance 
	else if (-evalValue< MinEvalValue)
			return MinEvalValue;
		else return (int)(-evalValue);
}

int EndGame::evaluate(Configuration* config)
{
	float evalValue=0;

	if(IsWinning(config, config->blackNext))	
//	   if (config->amIBlack==config->blackNext) evalValue=2000;	// return the maximum value if winning
//	   else evalValue=-2000;					//	the opponent wins, return minimum value
		evalValue=MaxEvalValue;
	//	evaluate 5 factors according to the current configuration
	else
	{
		//	k= King_safety(config, config->blackNext);
		k= King_center(config, config->blackNext);
		m= Materials(config, config->blackNext);
		a= Attacking(config, config->blackNext);
		b= Board_control(config, config->blackNext);
		p= Pawn_position(config, config->blackNext);
		e= Ending_rules(config, config->blackNext);

		evalValue=100*(k+m+a+0.5*b+p);
	}
	
	if (config->amIBlack==config->blackNext) 
		if (evalValue> MaxEvalValue) 
			return MaxEvalValue;
			else return (int)(evalValue); //+0.0001 to avoid incorrect ignorance 
	else if (-evalValue< MinEvalValue)
			return MinEvalValue;
		else return (int)(-evalValue);
}


int Quiescence::evaluate(Configuration* config)
{
	float evalValue=0;

	m= Materials(config, config->blackNext);
	evalValue=100*m;
	
	if (config->amIBlack==config->blackNext) 
		if (evalValue> MaxEvalValue) 
			return MaxEvalValue;
			else return (int)(evalValue);
	else if (-evalValue< MinEvalValue)
			return MinEvalValue;
		else return (int)(-evalValue);
}
