#include "MinimaxTree.h"

#ifdef QUIESCENCE
#include "QuiescenceTree.h"
#endif //QUIESCENCE

#ifdef USINGHASH
#include "Hash.h"
#endif// USINGHASH

#ifdef DEBUG
#include <fstream>
#include <string>
#endif

using namespace std;


void cleanupSuccessors(ConfigList* successors)
{
	while(!successors->empty())
	{
		delete (Configuration*)(successors->front());
		successors->pop_front();
	}
}

MinimaxTree::MinimaxTree(Configuration* curConf, int plyLimit, int nodeLimit, Timer* pTimer)	
// constructor
{
	current_Conf=curConf;		//	save parameters to local variables
	ply_Limit=plyLimit;
	node_Limit=nodeLimit;
	nodeNum=0;
	m_pTimer=pTimer;

#ifdef DEBUG
	counter1=0;
	counter2=0;
	counter3=0;
#endif
}


MinimaxTree::~MinimaxTree()
{
	
}


int MinimaxTree::getnodeNum()	//	get the number of node searched
{
	return nodeNum;				
}

int MinimaxTree::ABP(Configuration* N, Configuration* B_Child, int A, int B, int Max_node, int plyNum)
// Alpha-Beta pruning
{
	//if current node is good or bad enough, don't expand
	if(MaxEvalValue==N->evalValue||MinEvalValue==N->evalValue)
	{
		return N->evalValue;
	}

	
#ifdef USINGTIMER
	//if time is out, return immediately
	if(m_pTimer->OutofTime())
	{
		if (Max_node==1)
		{
			N->backup_value=N->evalValue;
			
			if(A>N->backup_value)
			{
				return A;
			}
			else
			{
				return N->backup_value;
			}
		}
		else //for min node
		{
			N->backup_value=N->evalValue;
			
			if(B<N->backup_value)
			{
				return B;
			}
			else
			{
				return N->backup_value;
			}

		}
	}
#endif

	//	determine if the ply of nodes expanded is beyond the upperlimit
	int ply_Num;

	//	determine if the number of nodes expanded is beyond the upperlimit	
	ply_Num=plyNum;

#ifdef QUIESCENCE
	if (ply_Limit!=0) 
		if (ply_Num>=ply_Limit)
			if (Max_node==1)
			{
				if((N->evalValue-N->parentEval)>=quiescenceThreshold||
					(N->evalValue-N->parentEval)<=-quiescenceThreshold) //not quiescence state, do secondary search
				{
					QuiescenceTree* q=new QuiescenceTree(N,quiescencePlyLimit);
					q->getResponse();
					delete q;
				}
				else //quiescence state
				{
					N->backup_value=N->evalValue;
				}

				if(A>N->backup_value)
				{
					return A;
				}
				else
				{
					return N->backup_value;
				}
			}
			else //for min node
			{
				if((N->evalValue-N->parentEval)>=quiescenceThreshold||
					(N->evalValue-N->parentEval)<=-quiescenceThreshold) //not quiescence state, do secondary search
				{
					QuiescenceTree* q=new QuiescenceTree(N,quiescencePlyLimit);
					q->getResponse();
					delete q;
				}
				else //quiescence state
				{
					N->backup_value=N->evalValue;
				}

				if(B<N->backup_value)
				{
					return B;
				}
				else
				{
					return N->backup_value;
				}

			}
#else
	if (ply_Limit!=0) 
		if (ply_Num>=ply_Limit)
			if (Max_node==1)
			{
				N->backup_value=N->evalValue;
				
				if(A>N->backup_value)
				{
					return A;
				}
				else
				{
					return N->backup_value;
				}
			}
			else //for min node
			{
				N->backup_value=N->evalValue;
				
				if(B<N->backup_value)
				{
					return B;
				}
				else
				{
					return N->backup_value;
				}

			}
#endif

	ply_Num++;	//	increase the ply of node expanded

#ifdef USINGHASH
	KEY hashIndex=makeKey(N);
	KEY hashCheckNum=makeCheckNum(N);
	HashEntry* entry;
	bool isExact=false;

	entry=lookup(hashIndex,N->blackNext);
	int plyDepth=N->step_count+(ply_Limit-ply_Num+1);
	if(entry->checkNum==hashCheckNum && entry->plyDepth>=plyDepth)
	{
		#ifdef DEBUG
			nodeNum++;
			ofstream hashDebug("hashDebug.txt");
			hashDebug<<"hash="<<nodeNum<<endl;
		#endif
		switch(entry->entryType)
		{
			case EXACT:
			{
				#ifdef DEBUG
				hashDebug<<"exact="<<counter1++<<endl;
				#endif

				N->backup_value=entry->backup_value;
				return N->backup_value;
			}
			case LOWERBOUND:
			{
				if(entry->entryType>=B)
				{
					#ifdef DEBUG
					hashDebug<<"lowerbound="<<counter2++<<endl;
					#endif

					N->backup_value=entry->backup_value;
					return N->backup_value;
				}
				else
				{
					break;
				}
			}
			case UPPERBOUND:
			{
				if(entry->entryType<=A)
				{
					#ifdef DEBUG
					hashDebug<<"upperbound="<<counter3++<<endl;
					#endif

					N->backup_value=entry->backup_value;
					return N->backup_value;
				}
				else
				{
					break;
				}
			}
		}
	}
#endif

	int temp=-10;				

	ConfigList* N_Children=NULL;
 
	N_Children=N->successors();
	
	if(N_Children==NULL)
	{
		N->backup_value=N->evalValue;
		return N->backup_value;
	}

	if (N_Children->empty()) 
	{
		N->backup_value=0;
		
		delete N_Children;
		//return N->evaluate();	// if it is a leaf, return its evaluation
		return N->backup_value;
	}

	//to prevent three repetion, remove the node appeared before
	if(ply_Num==1) //if the first layer
	{
		if(N->evalValue>=0) //if I have advantage
		{
			ConfigList::iterator it=N_Children->begin();
			while(it != N_Children->end())
			{
				if((lastTwoConfig!=NULL) && lastTwoConfig->equal(*(*it)))
				{
					delete (*it);
					N_Children->remove(*it);
					break;
				}
				it++;
			}
		}
	}

	Configuration* child_best=new Configuration(N);

	if (Max_node==1)	//	if it is a Max node
	{
		do{	
			//Configuration *teeep=N_Children->front();
			temp=ABP(N_Children->front(), child_best, A, B, 0, ply_Num);	
			//	expand current child recursively, the next node is a Min node
			//	temp is the backup value of the current child
			if ((temp>N->backup_value)||(N->backup_value==MinEvalValue))
			{
				if(!N_Children->empty())
				{
					(*B_Child)= (*N_Children->front());
					B_Child->backup_value=temp;
					N->backup_value=temp;		
				}
				//	update the backup value of current node
			}
			if (temp>=B) //	if Alpha>Beta, return Beta
			{
				#ifdef USINGHASH
					entry->checkNum=hashCheckNum;
					entry->plyDepth=plyDepth;
					entry->entryType=LOWERBOUND;
					entry->backup_value=temp;
				#endif

				if (child_best!=NULL) delete child_best;
				cleanupSuccessors(N_Children);
				delete N_Children;
				return B;
			}
			if (temp>A)	 //	increase Alpha if possible
			{

				A=temp;		//	record the backup value of current node
				N->backup_value=temp;
				(*B_Child)= (*N_Children->front());
				#ifdef USINGHASH
					isExact=true;
				#endif
											//	record the current best child's configuration
			}
			delete (Configuration*)(N_Children->front());
			N_Children->pop_front();		//	move to the next child of N_Children
		}
		while (!N_Children->empty());		//	if all chirldren expanded, return
		
		#ifdef USINGHASH
			entry->checkNum=hashCheckNum;
			entry->plyDepth=plyDepth;
			if(isExact)
				entry->entryType=EXACT;
			else
				entry->entryType=UPPERBOUND;
			entry->backup_value=A;
		#endif

		if ((child_best!=NULL) && (ply_Num!=1)) delete child_best;	//	free pointer
		cleanupSuccessors(N_Children);
		delete N_Children;		
		return A;
	}
	else do		// it is a Min node
		{	
			//Configuration *teeep=N_Children->front();
			temp=ABP(N_Children->front(), child_best, A, B, 1, ply_Num);

			if ((temp<N->backup_value)||(N->backup_value==MinEvalValue))
			{
				if(!N_Children->empty())
				{
					(*B_Child)= (*N_Children->front());
					B_Child->backup_value=temp;
					N->backup_value=temp;	
				}
			}

			if (temp<=A) //	if Alpha>Beta, return Alpha
			{
				#ifdef USINGHASH
					entry->checkNum=hashCheckNum;
					entry->plyDepth=plyDepth;
					entry->entryType=UPPERBOUND;
					entry->backup_value=temp;
				#endif

				if (child_best!=NULL) delete child_best;
				cleanupSuccessors(N_Children);
				delete N_Children;				
				return A;
			}
			if (temp<B)	//	decrease Beta if possible
			{
				B=temp;
				N->backup_value=temp;
				(*B_Child)= (*N_Children->front());

				#ifdef USINGHASH
					isExact=true;
				#endif
			}
			delete (Configuration*)(N_Children->front());
			N_Children->pop_front();		//	move to the next child of N_Children
		}
		while (!N_Children->empty());

		#ifdef USINGHASH
			entry->checkNum=hashCheckNum;
			entry->plyDepth=plyDepth;
			if(isExact)
				entry->entryType=EXACT;
			else
				entry->entryType=LOWERBOUND;
			entry->backup_value=B;
		#endif

		//delete cur_child;
		if ((child_best!=NULL) && (ply_Num!=0)) delete child_best;
		cleanupSuccessors(N_Children);
		delete N_Children;				
		return B;
}


void MinimaxTree::getResponse(Configuration* lastTwoConfig)
{
	int Alpha=MinEvalValue;
	int Beta=MaxEvalValue;
	current_Conf->backup_value=MinEvalValue;
	Configuration* best_child=new Configuration(current_Conf);	

	this->lastTwoConfig=lastTwoConfig;

	ABP(current_Conf,	//	the imported current configuration
		best_child,		//	pointed to the best child (next) configuration after return
		Alpha,			//	A value
		Beta,			//	B value
		1,				//	Max node? Yes
		0				//	0 ply searched
		);	
	//	get the best child by Alpha-Beta pruning searching

	if (best_child!=NULL)
	{
		*current_Conf=*best_child;
		delete best_child;
	}
	return;
}

