// MoveGen.cpp: implementation of the MoveGen class.
//
//////////////////////////////////////////////////////////////////////
#include <queue>

#include "MoveGen.h"
#include "ConfigOperator.h"
#include "BitBoard.h"

using namespace std;
//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

MoveGen1::MoveGen1()
{

}

MoveGen1::~MoveGen1()
{

}

MoveList* MoveGen1::successors(Configuration* config)
{
	MoveList* pSuccessorList=new MoveList();

	//Suppose the necessary bitboards have been ready

	MOVE move;
	int from;
	int to;
	BITBOARD allOccupied=config->allOccupied;
	BITBOARD p;

	//pawn's move
	if(config->blackNext)
	{
		//Pawn 1 move
		p=(config->bP<<8) & (~allOccupied);
		while(p!=0)
		{
			to=pickFirst(p);
			from=to-8;
			move=(MOVE)((from<<8)+to);
			pSuccessorList->push_back(move);
			p&=~(Unit<<to);
		}

		//Pawn 2 move
		p=(config->bP<<16) & (~allOccupied) & (~(allOccupied<<8)&R4);
		while(p!=0)
		{
			to=pickFirst(p);
			from=to-16;
			move=(MOVE)((from<<8)+to);
			pSuccessorList->push_back(move);
			p&=~(Unit<<to);
		}

		//Pawn capture
		p=((config->bP&~C1)<<7)&config->wOccupied;
		while(p!=0)
		{
			to=pickFirst(p);
			from=to-7;
			move=(MOVE)((from<<8)+to);
			pSuccessorList->push_back(move);
			p&=~(Unit<<to);
		}
		
		p=((config->bP&~C8)<<9)&config->wOccupied;
		while(p!=0)
		{
			to=pickFirst(p);
			from=to-9;
			move=(MOVE)((from<<8)+to);
			pSuccessorList->push_back(move);
			p&=~(Unit<<to);
		}

		//En Passant
		p=((config->bP&~C1)<<7)&(config->flags.enPassant<<8);
		while(p!=0)
		{
			to=pickFirst(p);
			from=to-7;
			move=(MOVE)((from<<8)+to);
			pSuccessorList->push_back(move);
			p&=~(Unit<<to);
		}
		
		p=((config->bP&~C8)<<9)&(config->flags.enPassant<<8);
		while(p!=0)
		{
			to=pickFirst(p);
			from=to-9;
			move=(MOVE)((from<<8)+to);
			pSuccessorList->push_back(move);
			p&=~(Unit<<to);
		}
		//Promotion is not dealt with here, instead it's implemented in the Configuration::makeConfig

	}
	else
	{
		//Pawn 1 move
		BITBOARD p=(config->wP>>8) & (~allOccupied);
		while(p!=0)
		{
			to=pickFirst(p);
			from=to+8;
			move=(MOVE)((from<<8)+to);
			pSuccessorList->push_back(move);
			p&=~(Unit<<to);
		}

		//Pawn 2 move
		p=(config->wP>>16) & (~allOccupied) & (~(allOccupied>>8)&R5);
		while(p!=0)
		{
			to=pickFirst(p);
			from=to+16;
			move=(MOVE)((from<<8)+to);
			pSuccessorList->push_back(move);
			p&=~(Unit<<to);
		}

		//Pawn capture
		p=((config->wP&~C8)>>7)&config->bOccupied;
		while(p!=0)
		{
			to=pickFirst(p);
			from=to+7;
			move=(MOVE)((from<<8)+to);
			pSuccessorList->push_back(move);
			p&=~(Unit<<to);
		}
		
		p=((config->wP&~C1)>>9)&config->bOccupied;
		while(p!=0)
		{
			to=pickFirst(p);
			from=to+9;
			move=(MOVE)((from<<8)+to);
			pSuccessorList->push_back(move);
			p&=~(Unit<<to);
		}

		//En Passant
		p=((config->wP&~C8)>>7)&(config->flags.enPassant>>8);
		while(p!=0)
		{
			to=pickFirst(p);
			from=to+7;
			move=(MOVE)((from<<8)+to);
			pSuccessorList->push_back(move);
			p&=~(Unit<<to);
		}
		
		p=((config->wP&~C1)>>9)&(config->flags.enPassant>>8);
		while(p!=0)
		{
			to=pickFirst(p);
			from=to+9;
			move=(MOVE)((from<<8)+to);
			pSuccessorList->push_back(move);
			p&=~(Unit<<to);
		}
		
	}

	BITBOARD sideN;
	BITBOARD sideK;
	BITBOARD sideR;
	BITBOARD sideB;
	BITBOARD sideQ;
	BITBOARD sideOccupied;
	BITBOARD oppoOccupied;

	if(config->blackNext)
	{
		sideN=config->bN;
		sideK=config->bK;
		sideR=config->bR;
		sideB=config->bB;
		sideQ=config->bQ;
		sideOccupied=config->bOccupied;
		oppoOccupied=config->wOccupied;
	}
	else
	{
		sideN=config->wN;
		sideK=config->wK;
		sideR=config->wR;
		sideB=config->wB;
		sideQ=config->wQ;
		sideOccupied=config->wOccupied;
		oppoOccupied=config->bOccupied;
	}
	
	//Knight move or capture, need to check if column out of board (row don't need)
	p=((sideN&~C7&~C8)>>6)&~sideOccupied;
	while(p!=0)
	{
		to=pickFirst(p);
		from=to+6;
		move=(MOVE)((from<<8)+to);
		pSuccessorList->push_back(move);
		p&=~(Unit<<to);
	}
	p=((sideN&~C7&~C8)<<10)&~sideOccupied;
	while(p!=0)
	{
		to=pickFirst(p);
		from=to-10;
		move=(MOVE)((from<<8)+to);
		pSuccessorList->push_back(move);
		p&=~(Unit<<to);
	}
	p=((sideN&~C1&~C2)>>10)&~sideOccupied;
	while(p!=0)
	{
		to=pickFirst(p);
		from=to+10;
		move=(MOVE)((from<<8)+to);
		pSuccessorList->push_back(move);
		p&=~(Unit<<to);
	}
	p=((sideN&~C1&~C2)<<6)&~sideOccupied;
	while(p!=0)
	{
		to=pickFirst(p);
		from=to-6;
		move=(MOVE)((from<<8)+to);
		pSuccessorList->push_back(move);
		p&=~(Unit<<to);
	}
	p=((sideN&~C1)>>17)&~sideOccupied;
	while(p!=0)
	{
		to=pickFirst(p);
		from=to+17;
		move=(MOVE)((from<<8)+to);
		pSuccessorList->push_back(move);
		p&=~(Unit<<to);
	}
	p=((sideN&~C8)>>15)&~sideOccupied;
	while(p!=0)
	{
		to=pickFirst(p);
		from=to+15;
		move=(MOVE)((from<<8)+to);
		pSuccessorList->push_back(move);
		p&=~(Unit<<to);
	}
	p=((sideN&~C8)<<17)&~sideOccupied;
	while(p!=0)
	{
		to=pickFirst(p);
		from=to-17;
		move=(MOVE)((from<<8)+to);
		pSuccessorList->push_back(move);
		p&=~(Unit<<to);
	}
	p=((sideN&~C1)<<15)&~sideOccupied;
	while(p!=0)
	{
		to=pickFirst(p);
		from=to-15;
		move=(MOVE)((from<<8)+to);
		pSuccessorList->push_back(move);
		p&=~(Unit<<to);
	}
	
	//King
	if(sideK==0)
	{
		p=0;
	}
	else if((sideK&~C1)==0)
	{
		p=(sideK<<1)|(sideK<<8)|(sideK<<9)|(sideK>>7)|(sideK>>8);
	}
	else if((sideK&~C8)==0)
	{
		p=(sideK<<7)|(sideK<<8)|(sideK>>1)|(sideK>>8)|(sideK>>9);
	}
	else
	{
		p=(sideK<<1)|(sideK<<7)|(sideK<<8)|(sideK<<9)|(sideK>>1)|(sideK>>7)|(sideK>>8)|(sideK>>9);
	}
	p=p&~sideOccupied;
	from=pickFirst(sideK);
	while(p!=0)
	{
		to=pickFirst(p);
		move=(MOVE)((from<<8)+to);
		pSuccessorList->push_back(move);
		p&=~(Unit<<to);
	}

	//Rook
	p=sideR;
	BITBOARD p1;
	int pos;
	while(p!=0) //for each rook
	{
		pos=pickFirst(p);
		//get horizonal and vertical control
		p1=getHVControl(allOccupied,pos) & ~sideOccupied;
		from=pos;
		while(p1!=0)
		{
			to=pickFirst(p1);
			move=(MOVE)((from<<8)+to);
			pSuccessorList->push_back(move);
			p1&=~(Unit<<to);
		}
		p&=~(Unit<<from);
	}
	
	//Bishop
	p=sideB;
	while(p!=0)
	{
		pos=pickFirst(p);
		//get diagonal control
		p1=getXControl(allOccupied,pos) & ~sideOccupied;
		from=pos;
		while(p1!=0)
		{
			to=pickFirst(p1);
			move=(MOVE)((from<<8)+to);
			pSuccessorList->push_back(move);
			p1&=~(Unit<<to);
		}
		p&=~(Unit<<from);
	}

	//Queen
	p=sideQ;
	while(p!=0)
	{
		pos=pickFirst(p);
		//get horizonal and vertical control
		p1=getHVControl(allOccupied,pos) & ~sideOccupied;
		//get diagonal control
		p1=(getXControl(allOccupied,pos) & ~sideOccupied) | p1;
		from=pos;
		while(p1!=0)
		{
			to=pickFirst(p1);
			move=(MOVE)((from<<8)+to);
			pSuccessorList->push_back(move);
			p1&=~(Unit<<to);
		}
		p&=~(Unit<<from);
	}

	//Castling
	if(config->blackNext)
	{
		if(!config->flags.blackCastled && !config->flags.blackKingMoved) //not castling and king moved yet
		{
			if(!config->flags.leftBlackRookMoved && (allOccupied&0x0e)==0) //left rank not moved and no piece between them
			{
				if(0==config->valControl[0][1][2]&&0==config->valControl[0][2][2]&&0==config->valControl[0][3][2]&&0==config->valControl[0][4][2]) //no white control
				{
					move=0x0402;
					pSuccessorList->push_back(move);
				}
			}
			if(!config->flags.rightBlackRookMoved && (allOccupied&0x60)==0) //right rank not moved and no piece between them
			{
				if(0==config->valControl[0][4][2]&&0==config->valControl[0][5][2]&&0==config->valControl[0][6][2]) //no white control
				{
					move=0x0406;
					pSuccessorList->push_back(move);
				}
			}
		}
	}
	else
	{
		if(!config->flags.whiteCastled && !config->flags.whiteKingMoved) //not castling and king moved yet
		{
			if(!config->flags.leftWhiteRookMoved && (allOccupied&0x0e00000000000000)==0) //left rank not moved and no piece between them
			{
				if(0==config->valControl[7][1][0]&&0==config->valControl[7][2][0]&&0==config->valControl[7][3][0]&&0==config->valControl[7][4][0]) //no black control
				{
					move=0x3c3a;
					pSuccessorList->push_back(move);
				}
			}
			if(!config->flags.rightWhiteRookMoved && (allOccupied&0x6000000000000000)==0) //right rank not moved and no piece between them
			{
				if(0==config->valControl[7][4][0]&&0==config->valControl[7][5][0]&&0==config->valControl[7][6][0]) //no black control
				{
					move=0x3c3e;
					pSuccessorList->push_back(move);
				}
			}
		}
	}

	return pSuccessorList;
}

//MoveGen 2 come here

MoveGen2::MoveGen2()
{
	m_pMoveGen=new MoveGen1();
}

MoveGen2::~MoveGen2()
{
	if(NULL==m_pMoveGen)
	{
		delete m_pMoveGen;
	}
}

ConfigList* MoveGen2::successors(Configuration* config)
{
	priority_queue<Configuration*, vector<Configuration*>, CompareConfig> filterQueue;

	CompareConfig compareConfig;
	//select best/worst SelectNumber successors from all the successors 
	MoveList* allSuccessors=m_pMoveGen->successors(config);
	MoveList::iterator t;
	for(t=allSuccessors->begin(); t != allSuccessors->end();t++)
	{
		Configuration* tmpConfig=config->makeConfig(*t);
		tmpConfig->evaluate();
		if(filterQueue.size()<SelectNumber)
		{
			filterQueue.push(tmpConfig);
		}
		else
		{
			if(!compareConfig.compare(tmpConfig,(Configuration*)filterQueue.top()))
			{
				delete tmpConfig;
				continue;
			}
			else
			{
				Configuration* tmp=filterQueue.top();
				filterQueue.pop();
				filterQueue.push(tmpConfig);
				delete tmp;
			}
		}
	}
	
	delete allSuccessors;

	//put all the selected successors in the list
	ConfigList* successorList=new ConfigList();

	while(!filterQueue.empty())
	{
		successorList->push_front(filterQueue.top());
		filterQueue.pop();
	}

	return successorList;
}


//MoveGen3 comes here
MoveGen3::MoveGen3()
{

}

MoveGen3::~MoveGen3()
{

}

MoveList* MoveGen3::successors(Configuration* config)
{
	MoveList* pSuccessorList=new MoveList();

	//Suppose the necessary bitboards have been ready

	MOVE move;
	int from;
	int to;
	BITBOARD allOccupied=config->allOccupied;
	BITBOARD p;

	//pawn's move
	if(config->blackNext)
	{
		//Pawn capture
		p=((config->bP&~C1)<<7)&config->wOccupied;
		while(p!=0)
		{
			to=pickFirst(p);
			from=to-7;
			move=(MOVE)((from<<8)+to);
			pSuccessorList->push_back(move);
			p&=~(Unit<<to);
		}
		
		p=((config->bP&~C8)<<9)&config->wOccupied;
		while(p!=0)
		{
			to=pickFirst(p);
			from=to-9;
			move=(MOVE)((from<<8)+to);
			pSuccessorList->push_back(move);
			p&=~(Unit<<to);
		}

		//En Passant
		p=((config->bP&~C1)<<7)&(config->flags.enPassant<<8);
		while(p!=0)
		{
			to=pickFirst(p);
			from=to-7;
			move=(MOVE)((from<<8)+to);
			pSuccessorList->push_back(move);
			p&=~(Unit<<to);
		}
		
		p=((config->bP&~C8)<<9)&(config->flags.enPassant<<8);
		while(p!=0)
		{
			to=pickFirst(p);
			from=to-9;
			move=(MOVE)((from<<8)+to);
			pSuccessorList->push_back(move);
			p&=~(Unit<<to);
		}
		//Promotion is not dealt with here, instead it's implemented in the Configuration::makeConfig

	}
	else
	{
		//Pawn capture
		p=((config->wP&~C8)>>7)&config->bOccupied;
		while(p!=0)
		{
			to=pickFirst(p);
			from=to+7;
			move=(MOVE)((from<<8)+to);
			pSuccessorList->push_back(move);
			p&=~(Unit<<to);
		}
		
		p=((config->wP&~C1)>>9)&config->bOccupied;
		while(p!=0)
		{
			to=pickFirst(p);
			from=to+9;
			move=(MOVE)((from<<8)+to);
			pSuccessorList->push_back(move);
			p&=~(Unit<<to);
		}

		//En Passant
		p=((config->wP&~C8)>>7)&(config->flags.enPassant>>8);
		while(p!=0)
		{
			to=pickFirst(p);
			from=to+7;
			move=(MOVE)((from<<8)+to);
			pSuccessorList->push_back(move);
			p&=~(Unit<<to);
		}
		
		p=((config->wP&~C1)>>9)&(config->flags.enPassant>>8);
		while(p!=0)
		{
			to=pickFirst(p);
			from=to+9;
			move=(MOVE)((from<<8)+to);
			pSuccessorList->push_back(move);
			p&=~(Unit<<to);
		}
		
	}

	BITBOARD sideN;
	BITBOARD sideK;
	BITBOARD sideR;
	BITBOARD sideB;
	BITBOARD sideQ;
	BITBOARD sideOccupied;
	BITBOARD oppoOccupied;

	if(config->blackNext)
	{
		sideN=config->bN;
		sideK=config->bK;
		sideR=config->bR;
		sideB=config->bB;
		sideQ=config->bQ;
		sideOccupied=config->bOccupied;
		oppoOccupied=config->wOccupied;
	}
	else
	{
		sideN=config->wN;
		sideK=config->wK;
		sideR=config->wR;
		sideB=config->wB;
		sideQ=config->wQ;
		sideOccupied=config->wOccupied;
		oppoOccupied=config->bOccupied;
	}
	
	//Knight capture, need to check if column out of board (row don't need)
	p=((sideN&~C7&~C8)>>6)&oppoOccupied;
	while(p!=0)
	{
		to=pickFirst(p);
		from=to+6;
		move=(MOVE)((from<<8)+to);
		pSuccessorList->push_back(move);
		p&=~(Unit<<to);
	}
	p=((sideN&~C7&~C8)<<10)&oppoOccupied;
	while(p!=0)
	{
		to=pickFirst(p);
		from=to-10;
		move=(MOVE)((from<<8)+to);
		pSuccessorList->push_back(move);
		p&=~(Unit<<to);
	}
	p=((sideN&~C1&~C2)>>10)&oppoOccupied;
	while(p!=0)
	{
		to=pickFirst(p);
		from=to+10;
		move=(MOVE)((from<<8)+to);
		pSuccessorList->push_back(move);
		p&=~(Unit<<to);
	}
	p=((sideN&~C1&~C2)<<6)&oppoOccupied;
	while(p!=0)
	{
		to=pickFirst(p);
		from=to-6;
		move=(MOVE)((from<<8)+to);
		pSuccessorList->push_back(move);
		p&=~(Unit<<to);
	}
	p=((sideN&~C1)>>17)&oppoOccupied;
	while(p!=0)
	{
		to=pickFirst(p);
		from=to+17;
		move=(MOVE)((from<<8)+to);
		pSuccessorList->push_back(move);
		p&=~(Unit<<to);
	}
	p=((sideN&~C8)>>15)&oppoOccupied;
	while(p!=0)
	{
		to=pickFirst(p);
		from=to+15;
		move=(MOVE)((from<<8)+to);
		pSuccessorList->push_back(move);
		p&=~(Unit<<to);
	}
	p=((sideN&~C8)<<17)&oppoOccupied;
	while(p!=0)
	{
		to=pickFirst(p);
		from=to-17;
		move=(MOVE)((from<<8)+to);
		pSuccessorList->push_back(move);
		p&=~(Unit<<to);
	}
	p=((sideN&~C1)<<15)&oppoOccupied;
	while(p!=0)
	{
		to=pickFirst(p);
		from=to-15;
		move=(MOVE)((from<<8)+to);
		pSuccessorList->push_back(move);
		p&=~(Unit<<to);
	}
	
	//King
	if(sideK==0)
	{
		p=0;
	}
	else if((sideK&~C1)==0)
	{
		p=(sideK<<1)|(sideK<<8)|(sideK<<9)|(sideK>>7)|(sideK>>8);
	}
	else if((sideK&~C8)==0)
	{
		p=(sideK<<7)|(sideK<<8)|(sideK>>1)|(sideK>>8)|(sideK>>9);
	}
	else
	{
		p=(sideK<<1)|(sideK<<7)|(sideK<<8)|(sideK<<9)|(sideK>>1)|(sideK>>7)|(sideK>>8)|(sideK>>9);
	}
	p=p&oppoOccupied;
	from=pickFirst(sideK);
	while(p!=0)
	{
		to=pickFirst(p);
		move=(MOVE)((from<<8)+to);
		pSuccessorList->push_back(move);
		p&=~(Unit<<to);
	}

	//Rook
	p=sideR;
	BITBOARD p1;
	int pos;
	while(p!=0) //for each rook
	{
		pos=pickFirst(p);
		//get horizonal and vertical control
		p1=getHVControl(allOccupied,pos) & oppoOccupied;
		from=pos;
		while(p1!=0)
		{
			to=pickFirst(p1);
			move=(MOVE)((from<<8)+to);
			pSuccessorList->push_back(move);
			p1&=~(Unit<<to);
		}
		p&=~(Unit<<from);
	}
	
	//Bishop
	p=sideB;
	while(p!=0)
	{
		pos=pickFirst(p);
		//get diagonal control
		p1=getXControl(allOccupied,pos) & oppoOccupied;
		from=pos;
		while(p1!=0)
		{
			to=pickFirst(p1);
			move=(MOVE)((from<<8)+to);
			pSuccessorList->push_back(move);
			p1&=~(Unit<<to);
		}
		p&=~(Unit<<from);
	}

	//Queen
	p=sideQ;
	while(p!=0)
	{
		pos=pickFirst(p);
		//get horizonal and vertical control
		p1=getHVControl(allOccupied,pos) & oppoOccupied;
		//get diagonal control
		p1=(getXControl(allOccupied,pos) & oppoOccupied) | p1;
		from=pos;
		while(p1!=0)
		{
			to=pickFirst(p1);
			move=(MOVE)((from<<8)+to);
			pSuccessorList->push_back(move);
			p1&=~(Unit<<to);
		}
		p&=~(Unit<<from);
	}

	return pSuccessorList;
}

//MoveGen 4 come here

MoveGen4::MoveGen4()
{
//	m_pMoveGen=new MoveGen3();
	m_pMoveGen=new MoveGen1();
}

MoveGen4::~MoveGen4()
{
	if(NULL==m_pMoveGen)
	{
		delete m_pMoveGen;
	}
}

ConfigList* MoveGen4::successors(Configuration* config)
{
	priority_queue<Configuration*, vector<Configuration*>, CompareConfig> filterQueue;

	CompareConfig compareConfig;
	//select best/worst SelectNumber successors from all the successors 
	MoveList* allSuccessors=m_pMoveGen->successors(config);
	MoveList::iterator t;
	for(t=allSuccessors->begin(); t != allSuccessors->end();t++)
	{
		Configuration* tmpConfig=config->makeConfig2(*t);
		tmpConfig->evaluate2();
		if(filterQueue.size()<SelectNumber2)
		{
			filterQueue.push(tmpConfig);
		}
		else
		{
			if(!compareConfig.compare(tmpConfig,(Configuration*)filterQueue.top()))
			{
				delete tmpConfig;
				continue;
			}
			else
			{
				Configuration* tmp=filterQueue.top();
				filterQueue.pop();
				filterQueue.push(tmpConfig);
				delete tmp;
			}
		}
	}
	
	delete allSuccessors;

	//put all the selected successors in the list
	ConfigList* successorList=new ConfigList();

	while(!filterQueue.empty())
	{
		successorList->push_front(filterQueue.top());
		filterQueue.pop();
	}

	return successorList;
}
