#include "QuiescenceTree.h"
#include "MinimaxTree.h"

#ifdef DEBUG1
#include <fstream>
#include <string>
using namespace std;
#endif


void cleanupSuccessors2(ConfigList* successors)
{
	while(!successors->empty())
	{
		delete (Configuration*)(successors->front());
		successors->pop_front();
	}
}

QuiescenceTree::QuiescenceTree(Configuration* curConf, int plyLimit, int nodeLimit)	
// constructor
{
	current_Conf=curConf;		//	save parameters to local variables
	ply_Limit=plyLimit;
	node_Limit=nodeLimit;
	nodeNum=0;
}


QuiescenceTree::~QuiescenceTree()
{
	
}


int QuiescenceTree::getnodeNum()	//	get the number of node searched
{
	return nodeNum;				
}

int QuiescenceTree::ABP(Configuration* N, Configuration* B_Child, int A, int B, int Max_node, int plyNum)
// Alpha-Beta pruning
{
	//if current node is good or bad enough, don't expand
	if(MaxEvalValue==N->evalValue||MinEvalValue==N->evalValue)
	{
		return N->evalValue;
	}

	//	determine if the ply of nodes expanded is beyond the upperlimit
	int ply_Num;

	//	determine if the number of nodes expanded is beyond the upperlimit	
	ply_Num=plyNum;
/*
	if (ply_Limit!=0) 
		if (ply_Num>=ply_Limit)
			if (Max_node==1)
			{
				N->backup_value=N->evalValue;
				
				if(A>N->backup_value)
				{
					return A;
				}
				else
				{
					return N->backup_value;
				}
			}
			else //for min node
			{
				N->backup_value=N->evalValue;
				
				if(B<N->backup_value)
				{
					return B;
				}
				else
				{
					return N->backup_value;
				}

			}
*/
	if ((ply_Limit!=0&&ply_Num>=ply_Limit)||					//to limit
		((N->evalValue-N->parentEval)<quiescenceThreshold&&		//or to stable state
		(N->evalValue-N->parentEval)>-quiescenceThreshold))
	{
			if (Max_node==1)
			{
				
				N->backup_value=N->evalValue;
				
				if(A>N->backup_value)
				{
					return A;
				}
				else
				{
					return N->backup_value;
				}
			}
			else //for min node
			{
				N->backup_value=N->evalValue;
				
				if(B<N->backup_value)
				{
					return B;
				}
				else
				{
					return N->backup_value;
				}

			}
	}

	ply_Num++;	//	increase the ply of node expanded

	int temp=-10;				
	
	ConfigList* N_Children=NULL;

	N_Children=N->successors2();

#ifdef DEBUG1
	string filename="debug";
	filename+='0'+ply_Num;
	filename+=".txt";
	ofstream debug(filename.c_str());
	ConfigList::iterator d_it=N_Children->begin();
	while(d_it != N_Children->end())
	{
		for(int i=0;i<8;i++)
		{
			for(int j=0;j<8;j++)
			{
				string tmp;
				switch ((*d_it)->board[i][j])
				{
				case wKing:		tmp="wK";
								break;
				case wQueen:	tmp="wQ";
								break;
				case wRook:		tmp="wR";
								break;
				case wBishop:	tmp="wB";
								break;
				case wKnight:	tmp="wN";
								break;
				case wPawn:		tmp="wP";
								break;
				case bKing:		tmp="bK";
								break;
				case bQueen:	tmp="bQ";
								break;
				case bRook:		tmp="bR";
								break;
				case bBishop:	tmp="bB";
								break;
				case bKnight:	tmp="bN";
								break;
				case bPawn:		tmp="bP";
								break;
				default:		tmp="--";
								break;
				}
				debug<<tmp<<" ";
			}
			debug<<endl;
		}
		debug<<endl<<(*d_it)->evalValue<<"----------------------"<<endl<<endl;
		debug<<endl<<"parent="<<(*d_it)->parentEval<<"----------------------"<<endl<<endl;
		d_it++;
	}

	debug<<"********************************"<<endl;
#endif

/*
	while(N_Children->size()!=0)
	{
		Configuration* te=N_Children->top();
		N_Children->pop();
	}
*/
	
	if(N_Children==NULL)
	{
		N->backup_value=N->evalValue;
		return N->evalValue;
	}

	if (N_Children->empty()) 
	{
		delete N_Children;
		//return N->evaluate();	// if it is a leaf, return its evaluation
		return N->evalValue;
	}

	Configuration* child_best=new Configuration(N);
	

	if (Max_node==1)	//	if it is a Max node
	{
		do{	
			//Configuration *teeep=N_Children->front();
			temp=ABP(N_Children->front(), child_best, A, B, 0, ply_Num);	
			//	expand current child recursively, the next node is a Min node
			//	temp is the backup value of the current child
			if ((temp>N->backup_value)||(N->backup_value==MinEvalValue))
			{
				if(!N_Children->empty())
				{
					(*B_Child)= (*N_Children->front());
					B_Child->backup_value=temp;
					N->backup_value=temp;		
				}
				//	update the backup value of current node
			}
			if (temp>=B) //	if Alpha>Beta, return Beta
			{
				if (child_best!=NULL) delete child_best;
				cleanupSuccessors2(N_Children);
				delete N_Children;
				return B;
			}
			if (temp>A)	 //	increase Alpha if possible
			{
				A=temp;		//	record the backup value of current node
				N->backup_value=temp;
				(*B_Child)= (*N_Children->front());
											//	record the current best child's configuration
			}
			delete (Configuration*)(N_Children->front());
			N_Children->pop_front();		//	move to the next child of N_Children
		}
		while (!N_Children->empty());		//	if all chirldren expanded, return
		
		if ((child_best!=NULL) && (ply_Num!=1)) delete child_best;	//	free pointer
		cleanupSuccessors2(N_Children);
		delete N_Children;
		return A;
	}
	else do		// it is a Min node
		{	
			//Configuration *teeep=N_Children->front();
			temp=ABP(N_Children->front(), child_best, A, B, 1, ply_Num);

			if ((temp<N->backup_value)||(N->backup_value==MinEvalValue))
			{
				if(!N_Children->empty())
				{
					(*B_Child)= (*N_Children->front());
					B_Child->backup_value=temp;
					N->backup_value=temp;	
				}
			}

			if (temp<=A) //	if Alpha>Beta, return Alpha
			{
				if (child_best!=NULL) delete child_best;
				cleanupSuccessors2(N_Children);
				delete N_Children;				
				return A;
			}
			if (temp<B)	//	decrease Beta if possible
			{
				B=temp;
				N->backup_value=temp;
				(*B_Child)= (*N_Children->front());
			}
			delete (Configuration*)(N_Children->front());
			N_Children->pop_front();		//	move to the next child of N_Children
		}
		while (!N_Children->empty());
		
		//delete cur_child;
		if ((child_best!=NULL) && (ply_Num!=0)) delete child_best;
		cleanupSuccessors2(N_Children);
		delete N_Children;				
		return B;
}


void QuiescenceTree::getResponse()
{
	int Alpha=MinEvalValue;
	int Beta=MaxEvalValue;
	current_Conf->backup_value=MinEvalValue;
	Configuration* best_child=new Configuration(current_Conf);	

	ABP(current_Conf,	//	the imported current configuration
		best_child,		//	pointed to the best child (next) configuration after return
		Alpha,			//	A value
		Beta,			//	B value
		1,				//	Max node? Yes
		0				//	0 ply searched
		);	
	//	get the best child by Alpha-Beta pruning searching

	if (best_child!=NULL)
	{
		*current_Conf=*best_child;
		delete best_child;
	}
	return;
}

